#  Copyright (c) 1997-2009
#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
#  http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 2, or (at your option) any
#  later version: http://www.gnu.org/licenses/gpl.txt.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#-----------------------------------------------------------------------------
#  $Project: polymake $$Id: Polygon.pm 9026 2009-01-09 18:03:47Z gawrilow $

use Visual::Wire;

###############################################################################
#
#  Visual::Polygon  - a flat polygon embedded in R^2 or R^3
#

package Visual::Polygon;

use Polymake::Struct (
   [ '@ISA' => 'PointSet' ],

   # the proper cyclic drawing order of Vertices
   [ '$Facet' => 'undef' ],

   #  "R G B"		to fill the inner area
   [ '$FacetColor' => 'get_RGB(#%)', default => '$Visual::Color::facets' ],

   [ '$FacetTransparency' => 'undef' ],

   #  "hidden"
   [ '$FacetStyle' => 'undef' ],

   #  "R G B"                the facet borders
   [ '$EdgeColor' => 'get_RGB(#%)', default => '$Visual::Color::edges' ],

   [ '$EdgeThickness' => 'undef' ],

   #  "hidden"
   [ '$EdgeStyle' => 'undef' ],
);

sub Facets : method {
   my $self=shift;
   [ defined($self->Facet || [ 0..$#{$self->Vertices} ]) ]
}

sub Closed { 0 }
sub FacetLabels { undef }
sub FacetNeighbors { undef }

declare %decorations=%Visual::PointSet::decorations;
@decorations{qw( FacetColor FacetTransparency FacetStyle EdgeColor EdgeThickness EdgeStyle )}=();


package Visual::DegeneratedPolygon;
use Polymake::Struct (
   [ '@ISA' => 'Polygon' ],
);

sub new {
   bless [ @{(pop)} ];
}

# make it compatible to Visual::Wire
sub all_edges {
   return new Wire::EdgeIterator([ [0,1] ]);
}

###############################################################################
#
#  Visual::Polygons - a collection of polygones sharing the same vertex set
#                     and making up a single geometric object
#		      (this makes a difference for some visualizers, esp. javaview.)

package Visual::Polygons;
use Polymake::Struct (
   [ '@ISA' => 'PointSet' ],
   [ '@Facets' => 'check_points(#%)', default => 'croak("Facets missing")' ],
   [ '@FacetNeighbors' => 'undef' ],
   [ '$FacetLabels' => 'unify_labels(#%)', default => '"hidden"' ],
   [ '$FacetColor' => 'unify_decor(#%)', merge => 'unify_decor(#%)', default => '$Visual::Color::facets' ],
   [ '$FacetTransparency' => 'unify_decor(#%)', merge => 'unify_decor(#%)', default => 'undef' ],
   [ '$FacetStyle' => 'unify_decor(#%)', merge => 'unify_decor(#%)', default => 'undef' ],
   [ '$EdgeColor' => 'get_RGB(#%)', default => '$Visual::Color::edges' ],
   [ '$EdgeThickness' => 'undef' ],
   [ '$EdgeStyle' => 'undef' ],
   [ '$Closed' => 'defined($this->FacetNeighbors) && $this->Dim==3' ],
);

declare %decorations=(%Visual::Polygon::decorations, FacetLabels => enum('hidden'));


1

# Local Variables:
# c-basic-offset:3
# End:
