/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ifndef _POLYMAKE_LP_CLIENT_H
#define _POLYMAKE_LP_CLIENT_H "$Project: polymake $$Id: lp_client.h 8183 2008-06-30 16:44:34Z gawrilow $"

#include <polymake.h>
#include <Matrix.h>
#include <Vector.h>
#include <linalg_exceptions.h>

namespace polymake { namespace polytope {

   template <typename Solver>
   void valid_point(perl::Object p, Solver& solver)
   {
      typedef typename Solver::coord_type coord_type;
      const Matrix<coord_type> H=p.give("FACETS | INEQUALITIES"),
                               E=p.lookup("AFFINE_HULL | EQUATIONS");
      const typename Solver::feasibility_solution F=solver.check_feasibility(H,E);
      p.take("FEASIBLE") << F.first;
      if (F.first)
	 p.take("VALID_POINT", perl::temporary) << F.second;
      else
	 p.take("VALID_POINT") << perl::undefined();
   }

   template <typename Solver>
   void solve_lp(perl::Object p, perl::Object lp, Solver& solver, bool maximize)
   {
      typedef typename Solver::coord_type coord_type;
      const Matrix<coord_type> H=p.give("FACETS | INEQUALITIES"),
                               E=p.lookup("AFFINE_HULL | EQUATIONS");
      const Vector<coord_type> Obj=lp.give("LINEAR_OBJECTIVE");
      try {
	 typename Solver::lp_solution S=solver.solve_lp(H, E, Obj, maximize);
	 lp.take(maximize ? "MAXIMAL_VALUE" : "MINIMAL_VALUE") << S.first;
	 lp.take(maximize ? "MAXIMAL_VERTEX" : "MINIMAL_VERTEX") << S.second;
	 p.take("FEASIBLE") << true;
      }
      catch (const unbounded&) {
	 if (maximize)
	    lp.take("MAXIMAL_VALUE") << std::numeric_limits<coord_type>::infinity();
	 else
	    lp.take("MINIMAL_VALUE") << -std::numeric_limits<coord_type>::infinity();
	 lp.take(maximize ? "MAXIMAL_VERTEX" : "MINIMAL_VERTEX") << perl::undefined();
	 p.take("FEASIBLE") << true;
      }
      catch (const linalg_error& error) {
	 lp.take(maximize ? "MAXIMAL_VALUE" : "MINIMAL_VALUE") << perl::undefined();
	 lp.take(maximize ? "MAXIMAL_VERTEX" : "MINIMAL_VERTEX") << perl::undefined();
	 p.take("FEASIBLE") << false;
      }
   }

} }

#endif // _POLYMAKE_LP_CLIENT_H

// Local Variables:
// mode:C++
// c-basic-offset:3
// End:
