/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: bound.cc 9054 2009-02-08 10:55:41Z joswig $"

#include <polymake.h>
#include <SparseMatrix.h>
#include <transform.h>

namespace polymake { namespace polytope {

   template <typename Scalar>
   perl::Object bound(perl::Object p_in)
   {
      const bool is_positive=p_in.give("POSITIVE");
      if (!is_positive)
	 throw std::runtime_error("polyhedron must be positive");

      perl::Object p_out(perl::ObjectType::construct<Scalar>("Polytope"));
      p_out.set_description() << "Bounded polytope transformed from " << p_in.name() << endl;

      const int d=p_in.give("AMBIENT_DIM");

      SparseMatrix<Scalar> tau=unit_matrix<Scalar>(d+1);
      tau.col(0).fill(1);
      transform(p_out, p_in, tau);

      p_out.take("BOUNDED") << true;
      return p_out;
   }

   UserFunctionTemplate4perl("# Category: Transforming a polyhedron\n"
			     "# Make a positive polyhedron bounded.\n"
			     "# Apply a projective linear transformation to a polyhedron mapping the far hyperplane\n"
			     "#  to the hyperplane spanned by the points $(1,0,...,0,1,0,...)$.\n"
			     "# The origin $(1,0,...,0)$ is fixed.\n"
			     "#\n"
			     "# The input polyhedron should be @see POSITIVE; i.e. no negative coordinates.\n",
			     "bound<Scalar> (Polytope<Scalar>)");
} }
