/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: cross.cc 7858 2008-01-02 22:33:38Z gawrilow $"

#include <Rational.h>
#include <SparseMatrix.h>
#include <IncidenceMatrix.h>
#include <CubeFacets.h>
#include <polymake.h>

namespace polymake { namespace polytope {

perl::Object cross(int d, const Rational& s)
{
   if (d < 1)
      throw std::runtime_error("cross : dimension d >= 1 required");
   if (s <= 0)
      throw std::runtime_error("cross : x >= 0 required");

   const int n_vertices=2*d;

   perl::Object p("Polytope<Rational>");
   p.set_description() << "cross-polytope of dimension " << d << endl;

   SparseMatrix<Rational> V(n_vertices,d+1);
   V.col(0).fill(1);
   int c=1;
   for (Entire< Rows< SparseMatrix<Rational> > >::iterator v=entire(rows(V)); !v.at_end(); ++v, ++c) {
      (*v)[c]=s;
      ++v;
      (*v)[c]=-s;
   }
   IncidenceMatrix<> VIF(1<<d, n_vertices);
   std::copy(CubeFacets<int>(d).begin(), cols(VIF).begin());

   p.take("AMBIENT_DIM") << d;
   p.take("DIM") << d;
   p.take("N_VERTICES")	<< n_vertices;
   p.take("VERTICES") << V;
   p.take("VERTICES_IN_FACETS") << VIF;
   p.take("BOUNDED") << true;
   p.take("CENTERED") << true;

   return p;
}

UserFunction4perl("# Category: Producing from scratch\n"
		  "# Produce a @a d-dimensional cross polytope.\n"
		  "# Regular polytope corresponding to the Coxeter group of type `B_{d-1} = C_{d-1}`.\n"
		  "#\n"
		  "# All coordinates are +/- @a scale or 0.  @a scale must be positive.\n"
		  "# Default value @a x=1.\n"
		  "# args: d, [ scale ]\n",
		  &cross, "cross($;$=1)");
} }
