/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: cube.cc 7858 2008-01-02 22:33:38Z gawrilow $"

#include <Rational.h>
#include <SparseMatrix.h>
#include <CubeFacets.h>
#include <IncidenceMatrix.h>
#include <polymake.h>

namespace polymake { namespace polytope {

perl::Object cube(int d, Rational x_up, Rational x_low)
{
   if (d < 1)
      throw std::runtime_error("cube: dimension d >= 1 required");
   if (!x_up) {
      x_up=1; x_low=0;
   } else {
      if (x_up <= x_low)
	 throw std::runtime_error("cube: x_up > x_low required");
      x_low.negate();
   }

   perl::Object p("Polytope<Rational>");
   p.set_description() << "cube of dimension " << d << endl;

   SparseMatrix<Rational> F(2*d,d+1);
   Rows< SparseMatrix<Rational> >::iterator f=rows(F).begin();
   for (int i=1; i<=d; ++i) {
      (*f)[0]=x_low;
      (*f)[i]=1;
      ++f;
      (*f)[0]=x_up;
      (*f)[i]=-1;
      ++f;
   }

   IncidenceMatrix<> VIF(2*d, 1<<d, CubeFacets<int>(d).begin());

   p.take("AMBIENT_DIM") << d;
   p.take("DIM") << d;
   p.take("FACETS") << F;
   p.take("VERTICES_IN_FACETS") << VIF;
   p.take("BOUNDED") << true;

   return p;
}

UserFunction4perl("# Category: Producing from scratch\n"
		  "# Produce a @a d-dimensional cube.\n"
		  "# Regular polytope corresponding to the Coxeter group of type `B_{d-1} = C_{d-1}`.\n"
		  "#\n"
		  "# The bounding hyperplanes are `x_i` <= @a x_up and `x_i` >= @a x_low.\n"
		  "# Default values: @a x_up=1, @a x_low=-@a x_up or 1 if @a x_up==0.\n"
		  "# args: d, [ x_up, [ x_low ] ]\n",
		  &cube, "cube($; $=1, $=-$_[1] )");
} }
