/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: edge_directions.cc 8962 2008-12-03 15:47:54Z gawrilow $"

#include <polymake.h>
#include <Rational.h>
#include <Graph.h>
#include <Vector.h>
#include <Matrix.h>

namespace polymake { namespace polytope {

template <typename Coords>
EdgeMap<Undirected, Vector<Coords> >
edge_directions(perl::Object g, const Matrix<Coords>& vertices)
{
   const Graph<> G = g.give("ADJACENCY");
   EdgeMap<Undirected, Vector<Coords> > directions(G);

   for (Entire< Edges< Graph<> > >::const_iterator e=entire(edges(G));  !e.at_end();  ++e) {
      const int n_from=e.from_node(), n_to=e.to_node();
      directions[*e]=vertices[n_to]-vertices[n_from];
   }
   return directions;
}

template <typename Coords>
EdgeMap<Undirected, Vector<Coords> >
edge_directions(perl::Object g, const Matrix<Coords>& vertices, const Set<int>& rays)
{
   const Graph<> G = g.give("ADJACENCY");
   EdgeMap<Undirected, Vector<Coords> > directions(G);

   for (Entire< Edges< Graph<> > >::const_iterator e=entire(edges(G));  !e.at_end();  ++e) {
      const int n_from=e.from_node(), n_to=e.to_node();
      if (rays.contains(n_from)) {
	 if (rays.contains(n_to))
	    directions[*e]=zero_vector<Coords>(vertices.cols());
	 else
	    directions[*e]=vertices[n_from];
      } else {
	 if (rays.contains(n_to))
	    directions[*e]=vertices[n_to];
	 else
	    directions[*e]=vertices[n_to]-vertices[n_from];
      }
   }
   return directions;
}

FunctionTemplate4perl("edge_directions(Graph Matrix Set)");
FunctionTemplate4perl("edge_directions(Graph Matrix)");

} }
