/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: gale_vertices.cc 7858 2008-01-02 22:33:38Z gawrilow $"

#include <algorithm>
#include <Rational.h>
#include <polymake.h>
#include <Vector.h>
#include <Matrix.h>
#include <linalg.h>
#include <random_generators.h>

/** @file gale_vertices
 *
 *  Calculate the coordinates of points for an affine Gale diagram.
 *  First the projection vector $(1, 1, ... )$ is tried, then random vectors.
 */

namespace polymake { namespace polytope {

   Matrix<double> gale_vertices(const Matrix<Rational>& G)
   {
      const int n=G.rows();
      UniformRNG<Rational> random(log2_ceil(n)+1);
      Vector<Rational> y(G.cols(), Rational(1)), G_y(G.rows());

      bool feasible;
      do {
	 G_y=G*y;
	 feasible=true;
	 for (Entire< Vector<Rational> >::iterator g_y=std::find_if(entire(G_y), operations::logical_not());
	      !g_y.at_end();  g_y=std::find_if(++g_y, operations::logical_not())) {
	    if (G[g_y - G_y.begin()].non_zero()) {
	       std::copy(translate(random, Rational(-1,2)).begin(), entire(y));
	       feasible=false;
	       break;
	    }
	 }
      } while (!feasible);

      Matrix<Rational> P=null_space(y);
      orthogonalize(entire(rows(P)));
      y /= sqr(y);

      Matrix<double> GV(G.rows(), G.cols());

      for (int i=0; i<n; ++i)
	 if ( (GV(i,0)=sign(G_y[i])) ) {
	    GV[i].slice(1)=P*(G[i]/G_y[i] - y);
	 }
      return GV;
   }

   Function4perl(&gale_vertices, "gale_vertices");
} }
