/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: hypersimplex.cc 8332 2008-08-19 14:31:49Z joswig $"

#include <hypersimplex.h>
#include <Rational.h>
#include <Matrix.h>
#include <PowerSet.h>

namespace polymake { namespace polytope {

perl::Object hypersimplex(int k, int d)
{
   if (d < 2)
      throw std::runtime_error("hypersimplex: dimension >= 2 required");
   if (k < 1 || k >= d)
      throw std::runtime_error("hypersimplex: 1 <= k < d required");

   perl::Object p("Polytope<Rational>");
   p.set_description() << "(" << k << "," << d << ")-hypersimplex" << endl;

   p.take("AMBIENT_DIM") << d;
   p.take("DIM") << d-1;

   // we already know the number of vertices
   const int n=Integer::binom(d,k);
   p.take("N_VERTICES") << n;

   Matrix<Rational> Vertices(n,d+1);
   Rows< Matrix<Rational> >::iterator v=rows(Vertices).begin();
   Subsets_of_k<sequence> enumerator(range(1,d), k);
   for (Subsets_of_k<sequence>::iterator s=entire(enumerator); !s.at_end(); ++s, ++v) {
      (*v)[0]=1;
      v->slice(*s).fill(1);
   }
   p.take("VERTICES") << Vertices;

   return p;
}

UserFunction4perl("# Category: Producing from scratch\n"
		  "# Produce the hypersimplex `&Delta;(k,d)`.\n"
		  "# The value of @a k defaults to 1, yielding a simplex.\n"
		  "# Note that the output is never full-dimensional.\n"
		  "# args: k,d \n",
		  &hypersimplex, "hypersimplex");
} }
