/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: included_polyhedra.cc 8429 2008-10-09 19:07:43Z gawrilow $"

#include <Rational.h>
#include <polymake.h>
#include <Matrix.h>

namespace polymake { namespace polytope {

   template <typename Coord>
   bool included_polyhedra(perl::Object p1, perl::Object p2, perl::OptionSet options)
   {
      const Matrix<Coord> vert=p1.give("VERTICES|POINTS"), ineq=p2.give("FACETS|INEQUALITIES"), eq=p2.give("AFFINE_HULL|EQUATIONS");

      for (typename Entire< Rows < Matrix<Coord> > >::const_iterator i=entire(rows(eq)); !i.at_end(); ++i)
	 for (typename Entire< Rows < Matrix<Coord> > >::const_iterator j=entire(rows(vert)); !j.at_end(); ++j)
	    if ((*i)*(*j)!=0) {
	       if (options["verbose"]) cout << "Equation " << *i << " not satisfied by point " << *j << endl;
	       return false;
	    }
      for (typename Entire< Rows < Matrix<Coord> > >::const_iterator i=entire(rows(ineq)); !i.at_end(); ++i)
	 for (typename Entire< Rows < Matrix<Coord> > >::const_iterator j=entire(rows(vert)); !j.at_end(); ++j)
	    if ((*i)*(*j)<0) {
	       if (options["verbose"]) cout << "Inequality " << *i << " not satisfied by point " << *j << endl;
	       return false;
	    }

      return true;
   }

   UserFunctionTemplate4perl("# category: Comparing\n"
			     "#  Tests if the first polyhedron is included in the second.\n"
			     "#  @author Sven Herrmann\n",
			     "included_polyhedra<Coord>(Polytope<Coord>, Polytope<Coord> { verbose => undef })");

   InsertEmbeddedRule("# category: Comparing\n"
		      "#  Tests if the two polyhedra are equal.\n"
		      "#  @author Sven Herrmann\n"
		      "user_function equal_polyhedra<Coord>(Polytope<Coord>, Polytope<Coord>)\n"
		      "{  return included_polyhedra(@_) && included_polyhedra(reverse @_)  }\n");
} }
