/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: rand01.cc 7976 2008-02-13 00:32:33Z gawrilow $"

#include <polymake.h>
#include <Rational.h>
#include <Matrix.h>
#include <random_generators.h>
#include <hash_set>

namespace polymake { namespace polytope {

perl::Object rand01(int d, int n, perl::OptionSet options)
{
   if (d<2 || n<=d || (n-1>>d)>=1)
      throw std::runtime_error("rand01 : 2 <= dim < #vertices <= 2^dim required");

   long seed;
   if (!(options["seed"] >> seed)) seed=random_seed();
   UniformRNG<Bitset> random(d,seed);

   perl::Object p("Polytope<Rational>");
   p.set_description() << "Random 0-1 polytope; seed=" << seed << endl;

   p.take("AMBIENT_DIM") << d;
   p.take("N_VERTICES") << n;

   hash_set<Bitset> bitvectors(n);

   while (bitvectors.size() < (size_t)n)
      bitvectors.insert(random.get());

   Matrix<Rational> V(n,d+1);
   hash_set<Bitset>::iterator bv=bitvectors.begin();
   for (Entire< Rows< Matrix<Rational> > >::iterator v=entire(rows(V)); !v.at_end(); ++v, ++bv)
      *v = 1 | same_element_sparse_vector<int>(*bv,d);
      
   p.take("VERTICES") << V;
   return p;
}

UserFunction4perl("# Category: Producing from scratch\n"
		  "# Produce a @a d-dimensional `0/1`-polytope with @a n random vertices.\n"
		  "# Uniform distribution.\n"
		  "# args: d, n, [ seed => VALUE ]\n",
		  &rand01, "rand01($$ { seed => undef })");
} }
