/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident"$Project: polymake $$Id: regular_subdivision.cc 8190 2008-07-01 09:13:29Z joswig $"

#include <polymake.h>
#include <Set.h>
#include <Matrix.h>
#include <IncidenceMatrix.h>
#include <Rational.h>
#include <Array.h>


namespace polymake { namespace polytope {
 
Array<Set <int> > regular_subdivision(const Matrix<Rational> &vertices, const Vector<Rational>& weight)
{
    //construct the lifted polytope
    const Matrix<Rational> lifted_vertices=vertices|weight;
    perl::Object p("Polytope<Rational>");
    p.take("VERTICES") << lifted_vertices;

    //we have to check for the case in which the subdivision is trivial
    const Matrix<Rational> aff_hull = p.give("AFFINE_HULL");
    const int last_col=aff_hull.cols()-1;

    const Vector<Rational> col = aff_hull.col(last_col);
    const Vector<Rational> n = zero_vector<Rational>(aff_hull.rows());
    if (col != n) {
      const Set<int> all=sequence(0,vertices.rows());
      return Array<Set <int> >(1,all);
    }
    
    const Matrix<Rational> facets=p.give("FACETS");
    const IncidenceMatrix<> vif=p.give("VERTICES_IN_FACETS");
    
    int n_simplices=0;
    for (int i=0; i<facets.rows(); ++i)
      if (facets(i,last_col)>0) ++n_simplices; //the lower facets are those with last coordinate>0
    
    Array< Set<int> > triang(n_simplices);
    
    Entire< Rows< IncidenceMatrix<> > >::const_iterator j=entire(rows(vif));
    int k=0;

    for (int i=0; i<facets.rows(); ++i, ++j)
       if (facets(i,last_col)>0) triang[k++]=*j; //the lower facets are those with last coordinate>0
 
    return triang;
}

   UserFunction4perl("# Category: Other\n"
		     "# Compute a regular subdivision of the polytope obtained\n"
		     "# by lifting @a points to @a weights and taking the lower\n"
		     "# complex of the resulting polytope.\n\n"
		     "# If the weight is generic the output is a triangulation.\n"
		     "# args: points weights\n"
		     "# @author Sven Herrmann\n",
		     &regular_subdivision,"regular_subdivision");
} }
