/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: revert.cc 9054 2009-02-08 10:55:41Z joswig $"

#include <polymake.h>
#include <transform.h>

namespace polymake { namespace polytope {

   template <typename Scalar>
   perl::Object revert(perl::Object p_in)
   {
      perl::Object p_out(perl::ObjectType::construct<Scalar>("Polytope"));
      p_out.set_description() << "Polytope reversely transformed from " << p_in.name() << endl;

      const Matrix<Scalar> RT_v=p_in.get_attachment("REVERSE_TRANSFORMATION");
      polytope::transform(p_out, p_in, RT_v, false);
      return p_out;
   }

   UserFunctionTemplate4perl("# Category: Transforming a polyhedron\n"
			     "# Apply a reverse transformation to a given polyhedron.\n"
			     "# All transformation clients keep track of the polytope's history.\n"
			     "# They write or update the attachment REVERSE_TRANSFORMATION.\n"
			     "#\n"
			     "# Applying revert to the transformed polytope re-constructs the original polyhedron.\n",
			     "revert<Scalar> (Polytope<Scalar>)");
} }
