/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: simplex.cc 7858 2008-01-02 22:33:38Z gawrilow $"

#include <polymake.h>
#include <Rational.h>
#include <Matrix.h>

namespace polymake { namespace polytope {

perl::Object simplex(int d, const Rational& s)
{
   if (d < 2)
      throw std::runtime_error("dimension must be at least 2");
   if (!s)
      throw std::runtime_error("scale must be non-zero");

   perl::Object p("Polytope<Rational>");
   p.set_description() << "simplex of dimension " << d << endl;

   p.take("AMBIENT_DIM") << d;
   p.take("DIM") << d;

   Matrix<Rational> V(d+1,d+1);
   V.col(0).fill(1);
   V.minor(range(1,d), range(1,d)).diagonal().fill(s);
   p.take("VERTICES") << V;

   p.take("N_VERTICES") << d+1;
   p.take("SIMPLICIALITY") << d;
   p.take("BOUNDED") << true;
   p.take("CENTERED") << false;

   return p;
}

UserFunction4perl("# Category: Producing from scratch\n"
		  "# Produce the standard @a d-simplex.\n"
		  "# Combinatorially equivalent to regular polytope corresponding to the Coxeter group of type `A_{d-1}`.\n"
		  "# args: d, [ scale ]\n",
		  &simplex, "simplex($;$=1)");
} }
