/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: tight_span.cc 8207 2008-07-08 16:17:54Z sherrmann $"

#include <polymake.h>
#include <Matrix.h>
#include <Rational.h>
#include <Vector.h>

namespace polymake { namespace polytope {

   perl::Object tight_span(const Matrix<Rational>& vertices, const Vector<Rational>& weight, bool dim)
   {
      Matrix<Rational> ineq;

      if (vertices.rows()!=weight.size())
	throw std::runtime_error("Weight vector has the wrong dimension.");

      if (!dim) ineq = weight|vertices.minor(All,range(1,vertices.cols()-1));
      else ineq =  weight|vertices;

      perl::Object p("Polytope<Rational>");
      p.take("INEQUALITIES") << ineq;
      p.take("BOUNDED") << false;
      return p;
   }

   perl::Object tight_span2(perl::Object p_in)
   {
      const Matrix<Rational> verts=p_in.give("VERTICES");
      const Vector<Rational> w = p_in.give("WEIGHTS");
      const int dim = p_in.give("DIM");
      return tight_span(verts, w, dim==verts.cols()-1);
   }

   UserFunction4perl("# Compute the tight span dual to\n"
		     "# the regular subdivision obtained by lifting @a points to\n" 
		     "# @a weight and taking the lower\n"
		     "# complex of the resulting polytope. \n\n"
		     "# @a full hast to be set to true if the polytope is full-dimensional.\n"
		     "# args: points weight [full=true]\n"
		     "# @author Sven Herrmann\n",
		     &tight_span,"tight_span(Matrix Vector; $=1)");

   UserFunction4perl("# Compute the tight span dual to\n"
		     "# the regular subdivision of a polytope obtained by the\n" 
		     "# @see WEIGHTS and taking the lower\n"
		     "# complex of the resulting polytope. \n\n"
		     "# @author Sven Herrmann\n",
		     &tight_span2,"tight_span(Polytope)");
} }
