/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: voronoi.cc 7960 2008-02-08 08:38:19Z joswig $"

#include <polymake.h>
#include <Rational.h>
#include <Vector.h>
#include <Matrix.h>
#include <ListMatrix.h>

namespace polymake { namespace polytope {

Matrix<Rational>
voronoi(perl::Object p)
{
   const Matrix<Rational> sites=p.give("SITES");
   const int n=sites.rows(), d=sites.cols();
   Vector<Rational> min_val=sites[0];
   Vector<Rational> max_val=min_val;
   ListMatrix< Vector<Rational> > voronoi_ineq(0,d+1);
   Rational maxx(0);

   for (int i=0; i<n; ++i) {
      if (sites(i,0)!=1)
	 throw std::runtime_error("voronoi: input points must be finite");
      Vector<Rational> next_voronoi_ineq=unit_vector<Rational>(d+1,d);
      next_voronoi_ineq[0]=sqr(sites[i])-1;
      if (next_voronoi_ineq[0]>maxx) maxx=next_voronoi_ineq[0];
      for (int k=1; k<d; ++k) {
	 const Rational& x(sites(i,k));
	 if (x>max_val[k])
	    max_val[k]=x;
	 else if (x<min_val[k])
	    min_val[k]=x;
	 next_voronoi_ineq[k]=(-2)*x;
      }
      voronoi_ineq /= next_voronoi_ineq;
   }

   // since all SITES are required to be distinct, the defining inequalities are, in fact, facets;
   // e.g., for the computation of DELAUNAY_GRAPH it is essential that the first facets correspond
   // to the sites
   voronoi_ineq /= unit_vector<Rational>(d+1,0); // facet at infinity

   return voronoi_ineq;
}

UserFunctionTemplate4perl("# Compute the Voronoi polyhedron of a given set of points."
                          "# The polyhedron is always unbounded.  Introduce artificial cut facets later"
			  "# (e.g., for visualization); this must be done after the vertex have been computed.",
			  "voronoi(VoronoiDiagram)");

} }
