/* Copyright (c) 1997-2008
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: zonotope.cc 8902 2008-11-10 14:39:35Z joswig $"

#include <polymake.h>
#include <Rational.h>
#include <Matrix.h>

namespace polymake { namespace polytope {
  
Matrix<Rational> zonotope(const Matrix<Rational>& Zones)
{
   const int dim=Zones.cols();
   Matrix<Rational> Zonotope(1,dim);	// one zero-filled row
    
   for (Entire< Rows< Matrix<Rational> > >::const_iterator z=entire(rows(Zones)); !z.at_end(); ++z)
     Zonotope = ( Zonotope + repeat_row(*z, Zonotope.rows()) ) /
       ( Zonotope - repeat_row(*z, Zonotope.rows()) );
   Zonotope.col(0).fill(1);

   return Zonotope;
}

  UserFunction4perl("# Category: Producing from given data\n"
		    "# Produce the points of a zonotope from given @a vectors.\n"
		    "# The zonotope is obtained as the iterated Minkowski sum of all intervals $[-x,x]$,\n"
		    "# where $x$ ranges over the rows of a given matrix.\n"
		    "# args: Vectors \n",
		    &zonotope,"zonotope(Matrix<Rational>)");
} }
