/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: ch2d_3phases.cc 8263 2008-08-11 11:58:07Z joswig $"

#include <list>
#include <polymake.h>
#include <Array.h>
#include <Graph.h>

namespace polymake { namespace tropical {

   Array<int> ch2d_3phases(const int n, const Array< Array< Set<int> > >& Types, const Graph<>& G) {

   std::list<int> cyclic;

   // find lower left pseudovertex; its first type entry contains all generators
   int current=0;
   while (Types[current][0].size()<n) ++current;
   cyclic.push_back(current);

   // go through the three phases
   for (int phase=1; phase<=3; ++phase) {
      while (true) {
	 const int sector=phase%3, other_sector=(phase-1)%3;
	 Entire<Graph<>::out_edge_list>::const_iterator e=entire(G.out_edges(current));
	 int x=e.to_node(); ++e;
	 while (!e.at_end()) {
	    const int primary_compare=incl(Types[x][sector],Types[e.to_node()][sector]);
	    if (primary_compare<0 ||
		(primary_compare==0 && incl(Types[x][other_sector],Types[e.to_node()][other_sector])<0))
		x=e.to_node();
	    ++e;
	 }
	 if (incl(Types[current][sector],Types[x][sector])<=0) {
	    current=x;
	    cyclic.push_back(current);
	 } else
	    break;
      }
   }

   return Array<int>(cyclic);
}

UserFunction4perl("# List the pseudovertices of a 2d tropical polytope in the boundary\n"
                  "# in counter-clockwise cyclic order.\n"
 		  "# args: TropicalPolytope \n",
 		  &ch2d_3phases, "ch2d_3phases");


} }
