/* Copyright (c) 1997-2008
   Technische Universitaet Berlin, Germany
   Department of Mathematics,
   Research Group Algorithmic and Discrete Mathematics
   mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330 Boston, MA 02111-1307, USA.
*/

#ident "$Project: polymake $$Id: types.cc 8241 2008-08-04 17:03:06Z sherrmann $"

#include <polymake.h>
#include <Vector.h>
#include <Matrix.h>

namespace polymake { namespace tropical {

template <typename Matrix1, typename Matrix2, typename Coord>
Matrix<Coord> multiply(const GenericMatrix<Matrix1,Coord>& A, const GenericMatrix<Matrix2,Coord>& B)
{
   const int m=A.rows(), n=B.cols();
   if (A.cols()!=B.rows())
      throw std::runtime_error("tropical::multiply: dimension mismatch");
   Matrix<Coord> C(m,n);
   for (int i=0; i<m; ++i)
      for (int j=0; j<n; ++j)
	 C(i,j)=accumulate(A.row(i)+B.col(j),operations::min());
   return C;
}

template <typename Coord>
Matrix<Coord> dualize(const Matrix<Coord>& points, const Matrix<Coord>& generators)
{
  return multiply(-points,generators);
}

UserFunctionTemplate4perl("# Dualizes a point set with respect to the generators of a tropical polytope.\n"
			  "# The @a points are dualized with respect to the (rows of the) matrix @a generators.\n"
			  "#\tDevelin & Sturmfels, Tropical Convexity, Lemma 22.\n"
			  "# args: points generators",
			  "dualize<Coord>(Matrix<Coord> Matrix<Coord>)");
} }

