/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: extract_pseudovertices.cc 8934 2008-11-20 12:17:49Z gawrilow $"

#include <polymake.h>
#include <Vector.h>
#include <Rational.h>
#include <Matrix.h>
#include <Graph.h>

namespace polymake { namespace tropical {

   void extract_pseudovertices(perl::Object t, perl::Object p) {
   const Matrix<Rational> V=p.give("VERTICES"); // col(1) expected to be zero due to construction in trop2poly
   const Set<int> far_face=p.give("FAR_FACE");
   const Graph<> G=p.give("BOUNDED_GRAPH.ADJACENCY");
   const int trop_adim=t.give("AMBIENT_DIM");

   const Matrix<Rational> VV= V.minor(sequence(0,V.rows())-far_face,sequence(1,trop_adim+1)); // we use the 0 in col(1)

   t.take("PSEUDOVERTICES") << VV;
   t.take("PSEUDOVERTEX_GRAPH.ADJACENCY") << renumber_nodes(G);
}

UserFunction4perl("# Get the pseudovertices of a tropical polytope from the bounded subcomplex of the corresponding unbounded polyhedron.\n"
 		  "# args: trop_file poly_file \n",
 		  &extract_pseudovertices, "extract_pseudovertices(TropicalPolytope<Rational> polytope::Polytope<Rational>)");


} }
