/* Copyright (c) 1997-2008
   Technische Universitaet Berlin, Germany
   Department of Mathematics,
   Research Group Algorithmic and Discrete Mathematics
   mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330 Boston, MA 02111-1307, USA.
*/

#ident "$Project: polymake $$Id: minkowski_sum.cc 8414 2008-09-04 15:21:25Z gawrilow $"

#include <polymake.h>
#include <linalg.h>

namespace polymake { namespace tropical {

template <typename Scalar>
perl::Object minkowski_sum(const Scalar& lambda, perl::Object P, const Scalar& mu, perl::Object Q)
{
   const Matrix<Scalar> pointsP=P.give("VERTICES | POINTS"),
                        pointsQ=Q.give("VERTICES | POINTS");

   if (pointsP.cols() != pointsQ.cols())
      throw std::runtime_error("dimension mismatch");

   Matrix<Scalar> result(pointsP.rows()*pointsQ.rows(), pointsP.cols(),
			 entire(product(rows(pointsP+same_element_matrix(lambda,pointsP.rows(),pointsP.cols())),
					rows(pointsQ+same_element_matrix(mu,pointsQ.rows(),pointsQ.cols())),
					operations::min())));
   perl::Object PQ(perl::ObjectType::construct<Scalar>("TropicalPolytope"));
   PQ.set_description()<<"Tropical Minkowski sum of "<<P.name()<<" and "<<Q.name()<<endl;

   PQ.take("POINTS") << result;
   return PQ;
}
  
   UserFunctionTemplate4perl("# Produces the tropical polytope lambda*P+mu*Q, where * and + are tropical scalar multiplication\n"
			     "# and tropical addition, respectively.\n"
			     "# args: lambda P mu Q\n",
			     "minkowski_sum<Scalar>($ TropicalPolytope<Scalar> $ TropicalPolytope<Scalar>)");
} }
