/* Copyright (c) 1997-2008
   Technische Universitaet Berlin, Germany
   Department of Mathematics,
   Research Group Algorithmic and Discrete Mathematics
   mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330 Boston, MA 02111-1307, USA.
*/

#ident "$Project: polymake $$Id: planar_coords.cc 9000 2008-12-24 13:10:41Z gawrilow $"

#include <polymake.h>
#include <Vector.h>
#include <Matrix.h>
#include <linalg.h>

namespace polymake { namespace tropical {

Matrix<double> planar_coords(const Matrix<double>& M, perl::OptionSet options)
{
   const int n=M.rows(), d=M.cols();
   Matrix<double> planar_M(n,2);
   
   Matrix<double> directions(d,2);
   if (!(options["Directions"] >> directions))
      for (int k=0; k<d; ++k) {
	 directions(k,0) = sin(k*2*M_PI/d);
	 directions(k,1) = cos(k*2*M_PI/d);
      }

   for (int i=0; i<n; ++i)
      for (int k=0; k<d; ++k) {
	 planar_M(i,0) += M(i,k)*directions(k,0);
	 planar_M(i,1) += M(i,k)*directions(k,1);
      }

   return planar_M;
}

Function4perl(&planar_coords, "planar_coords(Matrix<Float> { Directions => undef })");

} }
