/* Copyright (c) 1997-2004
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: poly2trop.cc 8934 2008-11-20 12:17:49Z gawrilow $"

#include <polymake.h>
#include <list>
#include <Rational.h>
#include <Vector.h>
#include <Matrix.h>
#include <Array.h>

namespace polymake { namespace tropical {

perl::Object poly2trop(perl::Object p) {
  const bool bounded=p.give("BOUNDED");

  if (!bounded)
    throw std::runtime_error("input polyhedron not bounded");

  Matrix<Rational> V=p.give("VERTICES | POINTS");
  V.col(0)=zero_vector<Rational>(V.rows());
  
  perl::Object t("TropicalPolytope");
  t.set_description() << "Tropical polytope of " << p.name() <<endl;
 
  t.take("POINTS") << V;
  return t;
}

UserFunction4perl("# Takes an ordinary convex polytope and interprets it in tropical projective space.\n"
 		  "# args: poly_infile\n",
 		  &poly2trop, "poly2trop(polytope::Polytope)");

} }

