/* Copyright (c) 1997-2009			 -*- C -*-
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Project: polymake $$Id: Shell.xs 9089 2009-02-19 17:37:14Z gawrilow $"

#include "Ext.h"

#if PerlVersion>=5010
#endif

static inline
GV* retrieve_gv(pTHX_ OP *o, OP *const_op, SV **const_sv, PERL_CONTEXT *cx, PERL_CONTEXT *cx_bottom)
{
   GV *gv;
#ifdef USE_ITHREADS
   SV **saved_curpad=PL_curpad;
   PL_curpad=pm_perl_get_cx_curpad(aTHX_ cx, cx_bottom);
#endif
   gv=cGVOPo_gv;
   if (const_op) *const_sv=cSVOPx_sv(const_op);
#ifdef USE_ITHREADS
   PL_curpad=saved_curpad;
#endif
   return gv;
}

static
SV* compose_varname(pTHX_ OP *o, OP *const_op, SV **const_sv, const char prefix, PERL_CONTEXT *cx, PERL_CONTEXT *cx_bottom)
{
   GV *gv=retrieve_gv(aTHX_ o,const_op,const_sv,cx,cx_bottom);
   SV *varname=newSVpvf("%c%s::%.*s", prefix, HvNAME(GvSTASH(gv)), (int)GvNAMELEN(gv), GvNAME(gv));
   return sv_2mortal(varname);
}

SV* pm_perl_get_name_of_ret_var(pTHX)
{
   PERL_CONTEXT *cx_bottom=cxstack, *cx=cx_bottom+cxstack_ix;
   while (cx >= cx_bottom) {
      if (CxTYPE(cx)==CXt_SUB && !SkipDebugFrame(cx,0)) {
	 OP *o;
	 if (cx->blk_gimme != G_SCALAR) break;
	 o=ReturnsToOp(cx);
	 while (o->op_type == OP_LEAVE) o=o->op_next;
	 if (o->op_type != OP_LEAVESUB && o->op_type != OP_LEAVESUBLV) {
	    if (o->op_type == OP_GVSV  &&  o->op_next->op_type == OP_SASSIGN) {
	       GV *gv=retrieve_gv(aTHX_ o,0,0,cx,cx_bottom);
	       return sv_2mortal(newSVpvn(GvNAME(gv),GvNAMELEN(gv)));
	    }
	    break;
	 }
      }
      --cx;
   }
   return 0;
}

static
OP* convert_eval_to_sub(pTHX)
{
   CV *cv=cxstack[cxstack_ix].blk_sub.cv;
   OP *me=PL_op, *start=me->op_next, *root=CvROOT(cv);
   root->op_type=OP_LEAVESUB;
   root->op_ppaddr=PL_ppaddr[OP_LEAVESUB];
   CvSTART(cv)=start;
   return start;
}

/* Some of the following routines currently seem to be unneeded.
   They are preserved, however, for teaching purposes, and, who knows... */

MODULE = Polymake::Core::Shell		PACKAGE = Polymake::Core::Shell

PROTOTYPES: DISABLE

void
name_of_arg_var(arg_no)
   I32 arg_no;
PPCODE:
{
   PERL_CONTEXT *cx_bottom=cxstack, *cx=cx_bottom+cxstack_ix;
   PUSHs(&PL_sv_undef);	/* default answer */

   while (cx >= cx_bottom) {
      if (CxTYPE(cx)==CXt_SUB && !SkipDebugFrame(cx,0)) {
	 OP *o=cx->blk_oldcop->op_next;
	 if (o->op_type == OP_PUSHMARK) {
	    do {
	       o=o->op_sibling;
	       if (!o) goto Leave;
	    } while (--arg_no>=0);
	    if (o->op_type == OP_NULL) o=cUNOPo->op_first;
	    if (o->op_type == OP_GVSV) {
	       GV *gv=retrieve_gv(aTHX_ o,0,0,cx,cx_bottom);
	       SETs(sv_2mortal(newSVpvn(GvNAME(gv),GvNAMELEN(gv))));
	    }
	 }
Leave:	 break;
      }
      --cx;
   }
}

void
name_of_ret_var()
PPCODE:
{
   SV *var_sv=pm_perl_get_name_of_ret_var(aTHX);
   if (var_sv)
      XPUSHs(var_sv);
   else
      XPUSHs(&PL_sv_undef);
}

void
name_of_custom_var(expect_assignment)
   I32 expect_assignment;
PPCODE:
{
   PERL_CONTEXT *cx_bottom=cxstack, *cx=cx_bottom+cxstack_ix;

   /* to keep things easier, only recognize assignments to whole arrays/hashes and hash element with literally given keys */

   while (cx >= cx_bottom) {
      if (CxTYPE(cx)==CXt_SUB && !SkipDebugFrame(cx,0)) {
	 OP *o=cx->blk_oldcop->op_next;
	 if (o->op_type == OP_PUSHMARK) {
	    o=o->op_sibling;
	    if (o) {
	       int allow_scalar=FALSE, allow_list=FALSE;
	       if (expect_assignment) {
		  switch (o->op_type) {
		  case OP_SASSIGN:
		     o=cBINOPo->op_last;			/* descend to the last SASSIGN operand = lhs term */
		     allow_scalar=TRUE;
		     break;
		  case OP_AASSIGN:
		     o=cLISTOPo->op_last;			/* descend to the last AASSIGN operand = lhs list */
		     o=cUNOPo->op_first->op_sibling;		/* pushmark, then lhs item: must be alone */
		     allow_list= o && !o->op_sibling;
		     break;
		  }
	       } else {
		  allow_scalar=allow_list=TRUE;
	       }
	       switch (o->op_type) {
	       case OP_NULL:
		  if (allow_scalar) {
		     o=cUNOPo->op_first;
		     if (o->op_type == OP_GVSV)
			XPUSHs(compose_varname(aTHX_ o,0,0,'$',cx,cx_bottom));
		  }
		  break;
	       case OP_HELEM:
		  if (allow_scalar) {
		     OP *key_op;
		     o=cUNOPo->op_first;			/* hash element: hash reference comes first */
		     if (o->op_type == OP_RV2HV) {
			key_op=o->op_sibling;
			if (key_op && key_op->op_type == OP_CONST) {
			   SV *key_sv;
			   XPUSHs(compose_varname(aTHX_ cUNOPo->op_first,key_op,&key_sv,'%',cx,cx_bottom));
			   XPUSHs(key_sv);
			}
		     }
		  }
		  break;
	       case OP_RV2AV:
		  if (allow_list)
		     XPUSHs(compose_varname(aTHX_ cUNOPo->op_first,0,0,'@',cx,cx_bottom));
		  break;
	       case OP_RV2HV:
		  if (allow_list)
		     XPUSHs(compose_varname(aTHX_ cUNOPo->op_first,0,0,'%',cx,cx_bottom));
		  break;
	       }
	    }
	 }
	 break;
      }
      --cx;
   }
}

void
get_chained(...)
PPCODE:
{
   PERL_CONTEXT *cx_bottom=cxstack, *cx=cx_bottom+cxstack_ix;
   while (cx >= cx_bottom) {
      if (CxTYPE(cx)==CXt_SUB && !SkipDebugFrame(cx,0)) {
	 OP *o=ReturnsToOp(cx);
	 cx->blk_gimme=G_ARRAY;
	 if (o->op_type != OP_LEAVESUB && o->op_type != OP_LEAVESUBLV) {
	    I32 skip=FALSE;
#ifdef USE_ITHREADS
	    SV **saved_curpad=0;
#endif
	    while (o->op_type == OP_CONST && o->op_next->op_type == OP_CONCAT) {
#ifdef USE_ITHREADS
	       if (!skip) {
		  saved_curpad=PL_curpad;
		  PL_curpad=pm_perl_get_cx_curpad(aTHX_ cx, cx_bottom);
	       }
#endif
	       XPUSHs(cSVOPo_sv);
	       skip=TRUE;
	       o=o->op_next->op_next;
	    }
	    if (skip) {
	       ReturnsToOp(cx)=o;
#ifdef USE_ITHREADS
	       PL_curpad=saved_curpad;
#endif
	    }
	    break;
	 }
      }
      --cx;
   }
}
items=0;

void
passed_to(sub)
   SV *sub;
PPCODE:
{
   PERL_CONTEXT *cx_bottom=cxstack, *cx_top=cx_bottom+cxstack_ix, *cx=cx_top;
   OP *o;
   while (cx >= cx_bottom) {
      if (CxTYPE(cx)==CXt_SUB && !SkipDebugFrame(cx,0)) {
	 o=ReturnsToOp(cx);
	 while (o->op_type == OP_LEAVE) o=o->op_next;
	 if (o->op_type != OP_LEAVESUB && o->op_type != OP_LEAVESUBLV) {
	    if (o->op_type == OP_GV && o->op_next->op_type == OP_ENTERSUB) {
	       CV *cv;
#ifdef USE_ITHREADS
	       SV **saved_curpad=PL_curpad;
	       PL_curpad=pm_perl_get_cx_curpad(aTHX_ cx, cx_bottom);
#endif
	       cv=GvCV(cGVOPo_gv);
#ifdef USE_ITHREADS
	       PL_curpad=saved_curpad;
#endif
	       if (cv == (CV*)SvRV(sub)) XSRETURN_YES;
	    }
	    break;
	 }
      }
      --cx;
   }
   XSRETURN_NO;
}

void
line_continued()
PPCODE:
{
   if (PL_lex_brackets==0 && PL_lex_state==LEX_NORMAL && PL_expect==XSTATE)
      XPUSHs(&PL_sv_undef);
   else
      XPUSHs(sv_2mortal(newSViv(CopLINE(&PL_compiling))));
}

void
_store_script()
PPCODE:
{
   /* We must convert a "one-shot" sub made for eval to a real persistent sub:
      1. Store the root operation (LEAVEEVAL) and increase its refcount,
         otherwise get destroyed in pp_require
      2. Prepare the special start operation converting the root to LEAVESUB,
         since all subsequent calls will be made via ENTERSUB.
	 This op will reside in an unused NULL enclosing this XSUB's call.
      3. Short-circuit this operation, making the first real op in the script
         the start one for all future calls.
      4. Provide CvDEPTH be decreased on exit, since LEAVEEVAL doesn't care about it.
   */
   CV *script_cv=cxstack[cxstack_ix].blk_eval.cv;
   OP *start=PL_op, *tmp_start=cUNOPx(start)->op_first, *root=PL_eval_root;
   while (start->op_type != OP_NEXTSTATE && start->op_type != OP_LEAVEEVAL) {
      start=start->op_next;
   }
   CvSTART(script_cv)=tmp_start;
   CvANON_on(script_cv);
   CvGV(script_cv)=(GV*)&PL_sv_undef;
   tmp_start->op_next=start;
   tmp_start->op_ppaddr=&convert_eval_to_sub;
   CvEVAL_off(script_cv);
   OP_REFCNT_LOCK;
   OpREFCNT_inc(root);
   OP_REFCNT_UNLOCK;
   CvROOT(script_cv)=root;
   XPUSHs(sv_2mortal(newRV((SV*)script_cv)));
   LEAVE;
   CvDEPTH(script_cv)=0;
   SAVELONG(CvDEPTH(script_cv));
   CvDEPTH(script_cv)=1;
   ENTER;
}

BOOT:
if (PL_DBgv) {
   CvNODEBUG_on(get_cv("Polymake::Core::Shell::name_of_arg_var", FALSE));
   CvNODEBUG_on(get_cv("Polymake::Core::Shell::name_of_ret_var", FALSE));
   CvNODEBUG_on(get_cv("Polymake::Core::Shell::name_of_custom_var", FALSE));
   CvNODEBUG_on(get_cv("Polymake::Core::Shell::get_chained", FALSE));
   CvNODEBUG_on(get_cv("Polymake::Core::Shell::passed_to", FALSE));
   CvNODEBUG_on(get_cv("Polymake::Core::Shell::line_continued", FALSE));
   CvNODEBUG_on(get_cv("Polymake::Core::Shell::_store_script", FALSE));
}
