#  Copyright (c) 1997-2009
#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
#  http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 2, or (at your option) any
#  later version: http://www.gnu.org/licenses/gpl.txt.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#-----------------------------------------------------------------------------
# $Project: polymake $$Id: CPlusPlus.pm 9026 2009-01-09 18:03:47Z gawrilow $

use namespaces;

package Polymake::Core::CPlusPlus;

my $debug= $DebugLevel ? $ENV{POLYMAKE_DEBUG_CLIENTS} : "";
my $dl_suffix="$debug.$DynaLoader::dl_dlext";
my $corelib="$InstallArch/lib/poly$dl_suffix";

my ($Object_pseudo_proto, $Object_pseudo_options);

#######################################################################################

package Polymake::Core::CPlusPlus::FuncDescr;
use Polymake::Struct (
   '$wrapper',
   '$func_ptr',
   '$name | func_ptr_type',
   '$source_file',
   '$arg_types',
   '$auto_func',
);

#######################################################################################

package Polymake::Core::CPlusPlus::AutoFunction;

use Polymake::Struct (
   [ new => '$$$%' ],
   '$name',
   [ '$perl_name' => '#1' ],
   [ '$wrapper_name' => '#1' ],
   [ '$flags' => '#2' ],
   [ '$application' => '#3' ],
   [ '$extension' => '$Application::extension' ],
   '@include',
   [ '$embedded' => 'undef' ],
   [ '$macro_call' => '$this->flags & $func_is_void ? "Void( " : "( "' ],
   '@all_params',
   '@template_params',
   '@extra_template_params',
   '@lvalue_params',
   [ '$via_object' => 'undef' ],
   [ '$opt_hashes' => '0' ],
   '@args_for_deduction',
   '%inst_cache',
   '%seen_in',
);

sub prepare {
   my ($self, $signature, $pkg, $app)=@_;
   my $internal=@_<4  and  $pkg="Polymake";
   my ($tpcount, $arg_off, $type_eval_pkg)=(defined($self->via_object)+0, 0, $pkg);
   my $tpcount_before_explicit=$tpcount;
   my $type_eval_symtab;
   my (@arg_list, $min, $defaults, @repeated);	# as in Overload::parse_signature
   my $fixed=1;
   my ($repeated_seen, $arg_for_lvalue_opt);

   if (is_string($self->extra_template_params)) {
      $type_eval_pkg="Polymake::Core::CPlusPlus::AutoFunction::_type_eval";
      $type_eval_symtab=get_pkg($type_eval_pkg,1);
      namespaces::using($type_eval_pkg,$pkg);
      push @arg_list, 'namespaces::TemplateExpression';
      push @repeated, undef;
      $arg_off=1;

      (my $extra=$self->extra_template_params) =~ s/^\s+//;
      $self->extra_template_params=[ ];
      while ($extra =~ /\G($id_re) (?: \s*=\s* ($extd_type_re))? \s*(?:,\s*|$)/oxgc) {
	 my ($id, $default)=($1,$2);
	 my $placeholder=_new PropertyParamedType($id,undef,undef);
	 $placeholder->cppoptions=new TemplateParam("T$tpcount",$tpcount++,$default);
	 $placeholder->abstract=sub { 1 };
	 define_function($type_eval_symtab, $id, sub { $placeholder->cppoptions->deduced ||= defined($min)+1; $placeholder });
	 push @{$self->extra_template_params}, $placeholder;
      }
   } elsif (@{$self->extra_template_params} or $self->flags & $func_has_prescribed_ret_type) {
      $tpcount+=@{$self->extra_template_params};
      $arg_off=1;
   }

   if ($self->perl_name ne "operator" && $self->perl_name ne "construct") {
      $self->name ||= $self->perl_name;
      if ($tpcount > $tpcount_before_explicit) {
	 $self->name .= "<" . join(",", map { "T$_" } $tpcount_before_explicit..$tpcount-1) . ">";
      }
   }

   if ($self->flags & $func_is_method) {
      unless ($internal) {
	 my $opts=UNIVERSAL::can($pkg,"c++options");
	 if ($opts) {
	    if ($opts==\&ObjectType::cppoptions) {
	       $self->flags |= $func_is_top_object_method;
	       if ($self->via_object) {
		  push @{$self->all_params}, "self.";
		  $self->wrapper_name.="_O";
	       }
	       goto NOMETHOD;
	    } else {
	       $opts=$opts->();
	    }
	 } else {
	    my $proto=(UNIVERSAL::can($pkg,"prototype") || croak( "perl package $pkg has no C++ binding" ))->();
	    unless ($opts=$proto->cppoptions) {
	       croak( "class ", $proto->full_name, " has no C++ binding" );
	    }
	 }
	 if ($opts->builtin && !is_code($opts->builtin)) {
	    croak( "can't declare methods for C++ primitive type ", $opts->builtin );
	 }
      }
      push @{$self->all_params}, "arg0.get<T$tpcount>()".($self->perl_name ne 'operator' && ".");
      push @{$self->template_params}, [ $arg_off, $self->flags & $func_is_non_const ? $func_is_lvalue : $self->flags & ($func_is_lvalue|$func_is_lvalue_opt) ];
      $arg_for_lvalue_opt=$tpcount if $self->flags & ($func_is_lvalue | $func_is_lvalue_opt);
      ++$tpcount;
    NOMETHOD: ;
   }
   if ($self->flags & $func_has_prescribed_ret_type) { $self->wrapper_name.="_R"; }
      
   $signature =~ s/^\s+//;
   while (pos($signature) < length($signature)) {
      if ($signature =~ /\G; \s*/gxc) {
	 if (defined $min) {
	    croak( "invalid signature: multiple ';'" );
	 }
	 $min=@arg_list;
         next;
      }

      my ($as_template_arg, $wrapper_suffix);
      my $lval_flag=0;
      my $cpp_arg_no=@{$self->all_params};
      my $perl_arg_no=$cpp_arg_no+$arg_off;
      my $cpp_arg="arg$cpp_arg_no";

      if ($signature =~ /\G \@ \s*$/gxc) {
	 if ($repeated_seen) {
	    croak( "several repeating trailing lists not allowed" );
	 }
         $wrapper_suffix="e";
	 $self->flags |= $func_is_ellipsis;
	 undef $fixed;

      } elsif ($signature =~ /\G % \s*$/gxc) {
         $wrapper_suffix="o";
	 if (defined $min) {
	    # provide undef's as default values for the optional parameters
	    $#{$defaults ||= [ ]}=$#arg_list;
	 }
	 undef $fixed;

      } elsif ($signature =~ /\G (lvalue(_opt)? \(\s*)? (\*) (?(1) \s*\)) \s*(?:,\s*)? /gxc) {
	 if ($repeated_seen) {
	    croak( "repeating argument must be the last one in the signature" );
	 }
         $as_template_arg=1;
	 if ($2) {
	    $lval_flag=$func_is_lvalue_opt;
	    unless ($self->flags & ($func_is_lvalue | $func_is_lvalue_opt | $func_is_void)) {
	       $arg_for_lvalue_opt=$tpcount;
	       $self->flags |= $func_is_lvalue_opt;
	    }
	 } elsif ($1) {
	    $lval_flag=$func_is_lvalue;
	 }
	 push @arg_list, '$';
	 push @repeated, undef;

      } elsif ($signature =~ /\G (?: \$ | (lvalue(_opt)? \(\s*)? ($type_re) )
			         (?(1) \s*\)\s* | (?: \s*=\s* ($expression_re) | (?(3) \s*(\+)\s*) )?) \s*(?:,\s*)?/gxco) {

	 my ($lval, $lval_opt, $type, $default_value, $repeated)=($1, $2, $3, $4, $5);

	 if ($repeated_seen) {
	    croak( "repeating argument must be the last one in the signature" );
	 }
	 $repeated_seen=$repeated;
	 push @repeated, $repeated;

         if (defined($min) != defined($default_value)) {
            croak( defined($min) ? "optional argument without default value not allowed in a C++ function"
                                 : "only optional arguments may have default values" );
         }
	 if (defined($min) && defined($lval)) {
	    croak( "an lvalue argument can't be optional" );
	 }
	 if (defined $default_value) {
	    Overload::parse_default_value($defaults, $#arg_list, $pkg, $app, $default_value);
         }
         if (defined $type) {
	    if ($lval_opt) {
	       $lval_flag=$func_is_lvalue_opt;
	       unless ($self->flags & ($func_is_lvalue | $func_is_lvalue_opt | $func_is_void)) {
		  $arg_for_lvalue_opt=$tpcount;
		  $self->flags |= $func_is_lvalue_opt;
	       }
	    } elsif ($lval) {
	       $lval_flag=$func_is_lvalue;
	    }
	    my $proto;
            if ($type =~ /^$qual_id_re$/o) {
	       if ($type_eval_symtab && exists $type_eval_symtab->{$type}) {
		  my $placeholder=$type_eval_symtab->{$type}->();
		  push @{$self->args_for_deduction}, [ $placeholder, $perl_arg_no ];
		  push @arg_list, '$';
		  $as_template_arg=1;
	       } else {
		  $type=namespaces::lookup_class($pkg, $type, $app ? ($app->pkg) : ())
	                || croak( "Unknown type '$type'" );
		  push @arg_list, $type;
		  if (defined (my $opts=UNIVERSAL::can($type,"c++options"))) {
		     $as_template_arg= $opts != \&ObjectType::cppoptions;
		  } else {
		     $proto=(UNIVERSAL::can($type,"prototype") || croak( "pure perl package '$type' can't be passed to C++ subroutine" ))->();
		  }
	       }
	    } else {
	       translate_type(my $tt=$type);
	       $tt="package $type_eval_pkg; $tt";
	       $proto=$app ? $app->eval_expr->($tt) : eval($tt)  or report_type_error($type);
               push @arg_list, $proto->pkg;
	       if ($proto->abstract) {
		  if (defined($proto->context_pkg) && !($self->flags & $func_is_method) && $self->perl_name ne "construct") {
		     croak( "template parameters in abstract type ", $proto->full_name, " does not match the function declaration" );
		  }
		  push @{$self->args_for_deduction}, [ $proto, $perl_arg_no ];
		  $as_template_arg= $proto->cppoptions != $Object_pseudo_options;
		  undef $proto;
	       }
	    } 

	    if (defined($proto)) {
	       if (my $opts=$proto->cppoptions) {
		  if ($opts == $Object_pseudo_options) {
		     if ($lval) {
			croak( "superfluous 'lvalue' attribute: objects of type ", $proto->name, " are always passed by reference" );
		     }
		  } else {
		     if ($proto->cppoptions->builtin) {
			if ($lval) {
			   croak( $proto->full_name, " is declared as perl built-in type and can't be passed to C++ code by reference" );
			}
			$wrapper_suffix=$proto->cppoptions->builtin;
			$cpp_arg.=".get<$wrapper_suffix>()";
			$arg_list[-1]= $wrapper_suffix eq "int" ? "Polymake::Overload::integer" :
			               $wrapper_suffix eq "double" ? "Polymake::Overload::float" : '$';
		     } else {
			$as_template_arg=1;
		     }
		  }
	       } else {
		  croak( "class ", $proto->full_name, " has no C++ binding" );
	       }
            }
         } else {
            push @arg_list, '$';
         }
      } else {
         croak( "Invalid signature for a C++ function" );
      }

      if ($as_template_arg || $lval_flag==$func_is_lvalue_opt) {
         push @{$self->all_params}, "$cpp_arg.get<T$tpcount>()";
	 push @{$self->template_params}, [ $perl_arg_no, $lval_flag ];
	 ++$tpcount;
      } else {
         push @{$self->all_params}, $cpp_arg;
	 push @{$self->lvalue_params}, $perl_arg_no if $lval_flag;
      }
      $self->wrapper_name.="_".($wrapper_suffix || ($as_template_arg ? "X" : "x"));
      if ($lval_flag) { $self->wrapper_name.=$lval_flag; }
   }

   if ($type_eval_symtab) {
      Overload::finalize_extra_template_params($self->extra_template_params, $pkg, $type_eval_pkg, $app);
   } elsif (!$tpcount) {
      $self->inst_cache=undef;
   }

   if ($self->flags & ($func_is_lvalue|$func_is_lvalue_opt)) {
      $self->macro_call="Lvalue".$self->macro_call."T$arg_for_lvalue_opt, ";
   }
   if (my $fl=($self->flags & ($func_is_method|$func_is_lvalue|$func_is_lvalue_opt|$func_is_void))) {
      $self->wrapper_name .= "_f$fl";
   }
   if ($repeated_seen) {
      $self->flags |= $func_is_ellipsis;
      undef $fixed;
   }
   $self->wrapper_name =~ tr/:/_/;

   defined(wantarray) and do {
      $min=@arg_list unless defined $min;
      [ $min, $fixed && scalar(@arg_list), $defaults, \@repeated, @arg_list ]
   }
}

sub prepare_wrapper_code {
   my ($self, $code)=@_;
   my $n_args=@{$self->all_params};
   ++$n_args if @{$self->extra_template_params} || ($self->flags & $func_has_prescribed_ret_type);
   set_number_of_args($code, $n_args, $self->opt_hashes, $self->flags & $func_is_ellipsis, undef);
   declare_lvalue($code, 1) if $self->flags & ($func_is_lvalue|$func_is_lvalue_opt);
}

sub deduce_extra_params {
   my ($self, $args)=@_;
   Overload::deduce_extra_template_params($self->extra_template_params, $self->args_for_deduction,
					  $args, 0, $args->[0], \&guess_builtin_type);
}

sub gen_source_code {
   my ($self, $debug_code)=@_;

   my $t_args=$self->template_params ? $#{$self->extra_template_params}+1+$#{$self->template_params} : -1;
   ++$t_args if $self->via_object;
   my $typelist=join(",", map { "T$_" } 0..$t_args);
   my $change_guard_name=$self->name;
   my $call_name= $change_guard_name eq "stack" ? $self->application->name."::stack" : $change_guard_name;
   my $call_as_method= ($self->flags & $func_is_method) && (defined($self->via_object) || !($self->flags & $func_is_top_object_method));

   my @args=@{$self->all_params};
   my $arg_off=@{$self->extra_template_params} || ($self->flags & $func_has_prescribed_ret_type) ? 1 : 0;

   ( "   template <" . join(", ", $t_args>=0 ? (map { "typename T$_" } 0..$t_args) : "typename T0=void") . ">\n",
     "   FunctionInterface4perl( " . $self->wrapper_name . ($t_args>=0 && ", $typelist") . " ) {\n",
     $debug_code &&
     "      pm::break_on_throw(\"\");\n",
     @args>0 &&
     "      perl::Value " . join(", ", map { "arg$_(stack[".($_+$arg_off)."])" } 0..$#args) . ";\n",
     $self->via_object &&
     "      perl::Object arg0o;  arg0 >> arg0o;  const T0 self(arg0o);\n",
     "      WrapperReturn" . $self->macro_call . ($call_as_method ? shift(@args) : "") . "$call_name(" . join(", ", @args) . ") );\n",
     "   };\n",
     defined($self->embedded) && !$call_as_method && @args &&
     "  " . ( $change_guard_name =~ s/<.*>$// &&
	      " template <" . join(",", map { "typename X$_" } 0..$#{$self->extra_template_params}) . ">" ) .
     " int $change_guard_name(" . join(",", ("perl::Anything") x @args) . ");\n",
     "\n",
   )
}

sub find_instance {
   my ($self, $args)=@_;
   my $instance=$self->inst_cache;
   if (defined($self->via_object)) {
      $instance &&= $instance->{$self->via_object};
   }
   if (@{$self->extra_template_params}) {
      foreach my $proto ($self->deduce_extra_params($args)) {
	 if (!defined $proto) {
	    croak( "an explicit template parameter omitted in the call" );
	 }
	 $instance &&= $instance->{($proto->check_descr || return)->typeid};
      }
   }
   foreach (@{$self->template_params}) {
      my ($arg, $lval_flag)=($args->[$_->[0]], $_->[1]);
      my $typeid=get_magic_typeid($arg);
      if (!defined($typeid)) {
	 my $proto=guess_builtin_type($arg);
	 if ($lval_flag) {
	    croak( "Can't pass a primitive type ", $proto->full_name, " by reference to a C++ function" );
	 }
	 my $descr=($proto->cppoptions->descr ||= $proto->check_descr)
	    or return;
	 if (is_code($descr)) {
	    $descr=$descr->();
	 }
	 $typeid=$descr->typeid;
      }
      $instance &&= $instance->{$typeid};
      if ($lval_flag) {
	 if ($lval_flag == $func_is_lvalue_opt) {
	    $instance &&= $instance->[$typeid];
	 } elsif ($typeid==1) {
	    $self->complain_ro_violation($_->[0]);
	 }
      }
   }
   $self->check_lvalue_args($args);
   $instance;
}

sub disable_after_error { 0 }

sub name_for_disabled { &wrapper_name }
sub instance_macro { defined($_[0]->via_object) ? "MethodInstance4perl" : "FunctionInstance4perl" }

sub source_file { "auto-".$_[0]->perl_name.".cc" }

sub complain_ro_violation {
   my ($self, $arg_no)=@_;
   croak( "Attempt to modify a read-only C++ object passed as argument $arg_no to function ", $self->perl_name );
}

sub check_lvalue_args {
   my ($self, $args)=@_;
   foreach my $arg_no (@{$self->lvalue_params}) {
      if (defined (my $typeid=get_magic_typeid($args->[$arg_no]))) {
	 if ($typeid==1) {
	    complain_ro_violation($self,$arg_no);
	 }
      } else {
	 croak( "Argument $arg_no in the call to ", $self->perl_name, " can't be passed by reference\n",
		"as it is not an object with C++ binding." );
      }
   }
}

#######################################################################################

package Polymake::Core::CPlusPlus::LackingInstance;
use Polymake::Struct (
   [ new => '$$' ],
   [ '$is_instance_of' => '#1' ],
   '@args',	# 'C++ type expression', ...
   '@headers',	# [ 'header', ... ], ...
   [ '$embedded' => '#1 ->embedded' ],
   [ '$application' => '#1 ->application' ],
   [ '$extension' => '#1 ->extension' ],
   '$source_file',
);

sub new {
   my $self=&_new;
   my $args=pop;
   my $auto_func=$self->is_instance_of;

   if ($auto_func->via_object) {
      push @{$self->args}, $auto_func->via_object;
   }

   if (@{$auto_func->extra_template_params}) {
      my @extra=$auto_func->deduce_extra_params($args);
      foreach (@extra) { $_->check_descr }  # gather all lacking descriptions
      foreach (@extra) {
	 my ($arg_type, $includes, $embedded, $app, $extension)=$_->get_cpp_representation;
	 push @{$self->args}, $arg_type;
	 push @{$self->headers}, @$includes;
	 set_embedded($self,$embedded) if defined($embedded);
	 set_extension($self->extension,$extension) if defined($extension);
	 set_application($self->application,$app);
      }
   }

   foreach (@{$auto_func->template_params}) {
      my ($arg_no, $lval_flag)=@$_;
      my ($arg_type, $includes, $embedded, $app, $extension);
      if (defined (my $typeid=get_magic_typeid($args->[$arg_no]))) {
	 my $const= $lval_flag && $typeid==0 ? "" : "const ";
	 ($arg_type, $includes, $embedded, $app, $extension)=($root->typeids->{$typeid} || die "unregistered C++ typeid $typeid\n")->get_cpp_representation;
	 $arg_type="perl::Canned< $const$arg_type >";
      } else {
	 my $proto=guess_builtin_type($args->[$arg_no]);
	 my $opts=$proto->cppoptions->finalize;
	 if (ref($opts->builtin) eq "Polymake::Enum") {
	    $arg_type="perl::Enum<" . $opts->name . ">";
	 } elsif ($opts->template_params) {
	    $arg_type="perl::TryCanned< const ".$opts->name." >";
	 } else {
	    $arg_type=$opts->name;
	 }
	 $includes=$opts->include;
	 $app=$proto->application;
      }
      push @{$self->args}, $arg_type;
      push @{$self->headers}, @$includes;
      set_embedded($self,$embedded) if defined($embedded);
      set_extension($self->extension,$extension) if defined($extension);
      set_application($self->application,$app);
   }

   $self->source_file=
      defined($self->embedded)
      ? wrapper_filename($self->embedded)
      : ( defined($self->extension)
	  ? (is_object($self->extension) ? $self->extension : $self->extension->[0])->app_dir($self->application)
	  : $self->application->top
	) . "/src/perl/" . $auto_func->source_file($args);

   $self;
}

sub complain_source_conflict {
   "Don't know where to place the instance of ", $_[0]->is_instance_of->perl_name
}

sub wrapper_filename {
   my $ccfile=shift;
   $ccfile =~ s{[^/]+\.(?:cc|C|cpp|tcc)$}{perl/wrap-$&};
   $ccfile;
}

sub gen_source_code {
   my $self=shift;
   my $func=$self->is_instance_of;
   "   " . $func->instance_macro . "(" . join(", ", $func->wrapper_name, @{$self->args}) . ");\n"
}

sub gen_temp_code {
   my $self=shift;
   ( $self->is_instance_of->gen_source_code($debug), $self->gen_source_code );
}

sub include {
   my $self=shift;
   (@{$self->is_instance_of->include}, @{$self->headers})
}

sub temp_include {
   my $self=shift;
   if (defined (my $src_file=$self->is_instance_of->embedded || $self->embedded)) {
      my $app_name=$self->application->name;
      $src_file =~ s|^.*/apps/$app_name/src/|../src/|;
      ($src_file, @{$self->headers})
   } else {
      (@{$self->is_instance_of->include}, @{$self->headers})
   }
}

sub give_up {
   my $self=shift;
   if ($self->is_instance_of->disable_after_error($self)) {
      bless $self, "Polymake::Core::CPlusPlus::DisabledInstance";
      ()
   } else {
      (1)
   }
}

#######################################################################################

package Polymake::Core::CPlusPlus::DisabledInstance;
use Polymake::Struct ([ '@ISA' => 'LackingInstance' ]);

sub gen_source_code {
   my $self=shift;
   "   DisabledFunction4perl(" . join(", ", $self->is_instance_of->name_for_disabled, @{$self->args}) . ");\n"
}

sub give_up { 1 }

#######################################################################################

package Polymake::Core::CPlusPlus::LackingRegular;
use Polymake::Struct (
   [ new => '$$' ],
   [ '$is_instance_of' => '#1' ],
   '$func_ptr_type',
);

sub new {
   my $self=&_new;
   my $func_descr=pop;
   ($self->func_ptr_type=demangle($func_descr->func_ptr_type)) =~ s/pm::type2type<\s* ($balanced_re) \(\) \s* ($balanced_re) >/$1$2/xo;
   $self->func_ptr_type =~ s/(?<![\w:])pm::(perl::(?:Object(?:Type)?|OptionSet|Scalar|Array|Hash))(?![\w:])/$1/g;
   my $i=0;
   @{$self->is_instance_of->all_params}=map {
      my $arg="arg$i";
      if (defined (my $type_descr=$root->typeids->{$_})) {
	 my $proto=&{$type_descr->pkg->{prototype}}();
	 if (!$proto->cppoptions->builtin) {
	    my $const= !contains($self->is_instance_of->lvalue_params,$i) && "const ";
	    $arg.=".get< perl::TryCanned< $const".$type_descr->get_cpp_representation($proto)." > >()";
	 }
      }
      ++$i; $arg
   } @{$func_descr->arg_types};

   $self->is_instance_of->seen_in=1;
   $self;
}

sub gen_source_code {
   my ($self, $debug_code)=@_;
   my $func=$self->is_instance_of;
   my $args=$func->all_params;
   ( "   FunctionWrapper4perl( ".$self->func_ptr_type." ) {\n",
     $debug_code &&
     "      pm::break_on_throw(\"\");\n",
     @$args>0 &&
     "      perl::Value " . join(", ", map { "arg$_(stack[$_])" } 0..$#$args) . ";\n",
     "      IndirectWrapperReturn".($func->flags & $func_is_void ? "Void" : "")."(" . join(", ", @$args) . ");\n",
     "   }\n",
     "   FunctionWrapperInstance4perl( ".$self->func_ptr_type." );\n\n",
   )
}

sub gen_temp_code { gen_source_code(@_,$debug) }

sub include { () }

sub temp_include {
   my $self=shift;
   my $src_file=$self->is_instance_of->embedded;
   my $app_name=$self->is_instance_of->application->name;
   $src_file =~ s|^.*/apps/$app_name/src/|../src/|;
   $src_file
}

sub source_file { LackingInstance::wrapper_filename( (shift)->is_instance_of->embedded ) }
sub extension { (shift)->is_instance_of->extension }

sub give_up { 1 }

#######################################################################################

package Polymake::Core::CPlusPlus::Constructor;
use Polymake::Struct (
   [ '@ISA' => 'AutoFunction' ],
   [ new => '' ],
   [ '$perl_name' => '"construct"' ],
   [ '$wrapper_name' => '"new"' ],
   [ '$macro_call' => '"New(T0, "' ],
   [ '@extra_template_params' => '(0)' ],
);

sub disable_after_error {
   my ($self, $failed)=@_;
   @{$failed->args}<=2;
}

sub create_fallback {
   $#{$_[0]->all_params} ? sub { croak( "Class ", $_[0]->full_name, " has no default constructor" ) } : \&construct_any
}

sub deduce_extra_params {
   # the only template type argument is the type to be constructed,
   # already represented by a PropertyType object sitting at the front of the argument list
   $_[1]->[0]
}

sub source_file { $_[1]->[0]->name.".cc" }

#######################################################################################

package Polymake::Core::CPlusPlus::SpecialOperator;
use Polymake::Struct (
   [ '@ISA' => 'AutoFunction' ],
   [ new => '$' ],
   [ '$perl_name' => '"operator"' ],
   [ '$wrapper_name' => '#1' ],
   [ '$seen_in' => '1' ],
   [ '@all_params' => '(undef)' ],
   [ '@template_params' => '([1,0])' ],
   [ '@extra_template_params' => '(0)' ],
);

*deduce_extra_params=\&Constructor::deduce_extra_params;
*source_file=\&Constructor::source_file;

sub gen_source_code { "" }

sub create_fallback {
   my $is_assign=$_[0]->wrapper_name eq "assign";
   sub {
      croak( $is_assign ? "Assignment " : "Conversion ",
	     $_[0]->full_name,
	     $is_assign ? " = " : " ( ",
	     is_object($_[1]) ? $_[1]->type->full_name : ref($_[1]) || "'$_[1]'",
	     $is_assign ? "" : " ) ", " not defined" );
   }
}
sub instance_macro { "OperatorInstance4perl" }

#######################################################################################

package Polymake::Core::CPlusPlus::IndirectWrapper;
use Polymake::Struct (
   [ '$wrapper_name' => '".wrp"' ],
   '%inst_cache',
   [ '$seen_in' => '1' ],
   [ '$template_params' => '0' ],
   [ '$via_object' => 'undef' ],
);

#######################################################################################

package Polymake::Core::CPlusPlus::Operator;

# the following has to match the declarations in perl/wrappers.h

my @op_wrapper_class=qw( Unary_ Binary_ UnaryAssign_ BinaryAssign_ - Binary_ );

use Polymake::Struct (
   [ '@ISA' => 'AutoFunction' ],
   [ new => '$$$;$' ],
   [ '$seen_in' => '1' ],
   [ '$application' => 'undef' ],
   [ '$sign' => '#3' ],
   '&code',
);

sub new {
   my $self=&_new;
   my $special_flag=$_[3];
   substr($self->wrapper_name,0,1)=$op_wrapper_class[$self->flags];
   if ($self->flags & 1) {
      @{$self->template_params}=([0,$self->flags & ($func_is_lvalue|$func_is_lvalue_opt)], [1,0]);
      $#{$self->all_params}=1;
      if ($self->flags & $func_is_lvalue) {
	 # binary operation with assignment, operands swapping can't occur
	 if ($special_flag eq "F") {
	    $self->code=sub {
	       pop;
	       if (is_object($_[1]) || is_integer($_[1])) {
		  &{ resolve_auto_function($self,\@_) }
	       } else {
		  my @args=($_[0], defined(try_to_integer($_[1]) || CPlusPlus::std_parsing_constructor($_[0]->type,$_[1])));
		  resolve_auto_function($self,\@args)->(@args)
	       }
	    };
	 } else {
	    $self->code=sub { pop; &{ resolve_auto_function($self,\@_) } };
	 }
	 declare_lvalue($self->code, 1);
      } elsif ($self->flags & $func_is_lvalue_opt) {
	 # internal function mapped to an operator, no additional arguments
	 $self->code=sub { &{ resolve_auto_function($self,\@_) } };
	 declare_lvalue($self->code, 1);
      } else {
	 # binary operation, operands may be swapped
	 @{$self->template_params}=([0,0], [1,0]);
	 if ($special_flag eq "F") {
	    $self->code=sub {
	       if (pop) {
		  if (is_object($_[1]) || is_integer($_[1])) {
		     my @args=@_[1,0];
		     local_scalar($self->flags, $func_is_void);
		     resolve_auto_function($self,\@args)->(@args);
		  } else {
		     my @args=(defined(try_to_integer($_[1]) || CPlusPlus::std_parsing_constructor($_[0]->type,$_[1])), $_[0]);
		     resolve_auto_function($self,\@args)->(@args)
		  }
	       } else {
		  if (is_object($_[1]) || is_integer($_[1])) {
		     &{ resolve_auto_function($self,\@_) }
		  } else {
		     my @args=($_[0], defined(try_to_integer($_[1]) || CPlusPlus::std_parsing_constructor($_[0]->type,$_[1])));
		     resolve_auto_function($self,\@args)->(@args)
		  }
	       }
	    };
	 } else {
	    $self->code=sub {
	       if (pop) {
		  my @args=@_[1,0];
		  local_scalar($self->flags, $func_is_void);
		  resolve_auto_function($self,\@args)->(@args);
	       } else {
		  &{ resolve_auto_function($self,\@_) }
	       }
	    };
	 }
      }

   } else {
      # unary operation
      @{$self->template_params}=([0, $self->flags & ($func_is_lvalue|$func_is_lvalue_opt)]);
      $#{$self->all_params}=0;
      $self->code=sub { splice @_,-2; &{ resolve_auto_function($self,\@_) } };
      if ($self->flags & ($func_is_lvalue|$func_is_lvalue_opt)) {
	 declare_lvalue($self->code, 1);
      }
   }

   $self;
}

sub gen_source_code { "" }

sub source_file {
   my ($self, $args)=@_;
   $args->[ ($self->flags & $func_is_void) ? 1 : 0 ]->type->name . ".cc";
}

sub complain_ro_violation {
   my ($self)=@_;
   croak("Attempt to modify a read-only C++ object passed to operator ", $self->sign);
}

sub disable_after_error { 1 }
sub create_fallback { my $sign=$_[0]->sign; sub { missing_op(@_,0,$sign) } }
sub name_for_disabled { (shift)->perl_name }
sub instance_macro { "OperatorInstance4perl" }

#######################################################################################

package Polymake::Core::CPlusPlus::TypeDescr;
use Polymake::Struct (
   '$pkg',
   '$vtbl',
   '$typeid',
   '$kind',		# Enum class_is_*
   [ '$generated_by' => 'undef' ],	# FuncDesr of a function created such an object if it does not correspond to any declared property type;
   '$source_name',			#     or typeid of a container/composite which this class belongs as element to
   '$include',
   '$embedded',
   '$application',
   '$extension',
);

sub get_cpp_representation {
   my ($self, $proto)=@_;
   $self->source_name ||= do {
      if (defined($self->generated_by)) {
	 demangle($self->typeid);
      } else {
	 ($proto ||= &{$self->pkg->{prototype}}())->cppoptions->finalize->name ||= demangle($self->typeid);
      }
   } || do {
      my $typeid=$self->typeid;
      die <<".";
Can't obtain the C++ representation for typeid $typeid.
Probably the mangling rules are not guessed correctly.
Please investigate and put suitable regular expressions
in the custom array \@Polymake::Core::CPlusPlus::to_mangled_defconstr.
.
   };
   if (wantarray) {
      $self->include ||= do {
	 if (defined (my $gen=$self->generated_by)) {
	    my $parent;
	    unless (ref($gen)) {
	       do {
		  $parent=$root->typeids->{$gen} ||
		     croak( ($parent || $self)->source_name, " pretends to be a dependent type of an unregistered C++ class ", demangle($gen) );
		  $gen=$parent->generated_by;
	       } while (defined($gen) && !ref($gen));
	    }
	    if (ref($gen)) {
	       # it is a descriptor of an auto-function
	       my $auto_func=$gen->auto_func;
	       my @includes=@{$auto_func->include};
	       $self->embedded=$auto_func->embedded;
	       set_extension($self->extension,$auto_func->extension) if defined($auto_func->extension);
	       $self->application=$auto_func->application;
	       foreach my $arg_type (@{$gen->arg_types}) {
		  (undef, my ($includes, $embedded, $app, $extension))=$root->typeids->{$arg_type}->get_cpp_representation;
		  push @includes, @$includes;
		  set_embedded($self,$embedded) if defined($embedded);
		  set_extension($self->extension,$extension) if defined($extension);
		  set_application($self->application,$app);
	       }
	       \@includes
	    } else {
	       # it is a typeid of a container class
	       (undef, my $includes, $self->embedded, $self->application, $self->extension)=$parent->get_cpp_representation;
	       $includes
	    }
	 } else {
	    $self->application=($proto ||= &{$self->pkg->{prototype}}())->application;
	    my $includes=$proto->cppoptions->finalize->include;
	    set_extension($self->extension,$proto->cppoptions->extension) if defined($proto->cppoptions->extension);
	    $includes
	 }
      };
      ($self->source_name, $self->include, $self->embedded, $self->application, $self->extension);
   } else {
      $self->source_name;
   }
}

sub complain_source_conflict {
   "Don't know where to place the instantiation of ", $_[0]->source_name
}

# Sometimes a TypeDescr object is being passed instead of PropertyType 0-th argument to a converting constructor.
# The following methods make it compatible to PropertyType to the extent sufficient for
# resolve_auto_function and new LackingInstance.

sub check_descr { shift }

sub name { &{(shift)->pkg->{type}}->name }

#######################################################################################

package Polymake::Core::CPlusPlus::Options;
use Polymake::Struct (
   [ new => '%' ],
   '$name',
   '$append_template_params',
   '@include',
   [ '$builtin' => 'undef' ],
   [ '$special' => 'undef' ],
   '$template_params',
   '$operators',
   [ '$fields' => 'undef' ],
   [ '$default_constructor' => '1' ],
   [ '$finalize_with' => 'undef' ],
   [ '$descr' => 'undef' ],	# cached for builtins and persistent types
   [ '$extension' => '$Application::extension' ],
);

sub finalize {
   my $self=shift;
   if (defined (my $proto=$self->finalize_with)) {
      undef $self->finalize_with;
      my @incs=@{$self->include};
      my @t_params;
      set_extension($self->extension,$proto->extension) if defined($proto->extension);

      foreach (is_object($proto->param) ? $proto->param : @{$proto->param}) {
	 unless ($_->cppoptions) {
	    die "Can't create C++ binding for ", $proto->full_name, ": non-C++ parameter ", $_->name, "\n";
	 }
	 my $p_opts=$_->cppoptions->finalize;
	 push @t_params, $p_opts->name;
	 push @incs, @{$p_opts->include};
	 set_extension($self->extension,$p_opts->extension) if defined($p_opts->extension);
      }

      if (is_code($self->name)) {
	 $self->name= is_method($self->name) ? $self->name->($proto,@t_params) : $self->name->(@t_params);
      } elsif ($self->template_params eq "*") {
	 $self->name="";
      } elsif ($self->name =~ /%\d/) {
	 unshift @t_params, undef;	# shift the indexing
	 $self->name =~ s/%(\d+)/ $t_params[$1] /g;
      } else {
	 $self->name.="< " . join(", ", @t_params) . ($self->append_template_params && ", ".$self->append_template_params) . " >";
      }
      if (@incs>@{$self->include}) {
	 $self->include=[ uniq(@incs) ];
      }
   }
   $self;
}

sub complain_source_conflict {
   "Can't create C++ binding for ", $_[0]->name;
}

sub recognizing_template {
   my ($self, $pkg)=@_;
   my $typenames=join(", ", map { "typename T$_" } 0..$self->template_params-1);
   my $Ts=join(",", map { "T$_" } 0..$self->template_params-1);
   my $declared=$self->name;
   if ($declared =~ /%\d/) {
      $declared =~ s/%(\d+)/"T".($1-1)/ge;
   } else {
      $declared .= "<" . join(",", $Ts, grep { length } $self->append_template_params) . ">";
   }
   return <<".";
   template <typename T, $typenames>
   pm::perl::SV4recognize<pm::identical<T, $declared >::value>* recognize(bait*, T*, $declared*)
   {
      return pm::perl::get_type<pm::list($Ts)>("$pkg", pm::identical<T, $declared >());
   }

.
}

sub recognizing_class {
   my ($self, $pkg)=@_;
   my $declared=$self->name;
   return <<".";
   template <typename T>
   pm::perl::SV4recognize<pm::identical<T, $declared >::value>* recognize(bait*, T*, $declared*)
   {
      return pm::perl::get_type<void>("$pkg", pm::identical<T, $declared >());
   }

.
}

sub register_template {
   my ($self, $pkg)=@_;
   return <<".";
   ClassTemplate4perl("$pkg");
.
}

sub register_class {
   my ($self, $pkg)=@_;
   "   ".($self->builtin || $self->special ? "Builtin" : "Class")."4perl(\"$pkg\", ".$self->name.");\n"
}

sub source_path {
   my ($self, $app)=@_;
   defined($self->extension)
   ? (is_object($self->extension) ? $self->extension : $self->extension->[0])->app_dir($app)
   : $app->top;
}
#######################################################################################
# bind all Object types involved in template arg deduction to the C++ Object class

package Polymake::Core::CPlusPlus::ObjectOptions;
use Polymake::Struct (
   [ '@ISA' => 'Options' ],
);

sub finalize { shift }
sub Polymake::Core::ObjectType::cppoptions { $Object_pseudo_options }
sub descr : lvalue {
   croak("attempt to pass a polymake Object to a C++ function expecting something different from of perl::Object");
   $Object_pseudo_proto;
}

#######################################################################################

package Polymake::Core::CPlusPlus::TemplateParam;
use Polymake::Struct (
   [ '@ISA' => 'Overload::TemplateParam' ],
);

sub builtin { undef }
sub special { undef }
sub finalize { shift }
sub include { [ ] }
sub template_params { undef }

#######################################################################################

package Polymake::Core::CPlusPlus::EmbeddedRules;
use Polymake::Struct (
   [ new => '@' ],
   '$lastfile',
   [ '$preproc' => '0' ],
   [ '@lines' => '@' ],
);

*TIEHANDLE=\&new;

sub READLINE {
   my $self=shift;
   if ($self->preproc) {
      --$self->preproc;
      shift @{$self->lines};

   } elsif (!@{$self->lines}) {
      undef;

   } elsif ($self->lines->[0] =~ /^(\#line\s+(\d+))\s+"(.*)"/) {
      shift @{$self->lines};
      $.=$2-1;

      # perl seems confused when it encounters the same source file in the consecutive lines
      if ($3 eq $self->lastfile) {
	 "$1\n";
      } else {
	 $self->lastfile=$3;
	 unshift @{$self->lines}, "$1 \"$3\"\n";
	 ++$self->preproc;
	 "package application;\n";
      }

   } else {
      ++$.;
      $self->lines->[0] =~ s{^.*\n}{}m;
      if (!length($self->lines->[0])) { shift @{$self->lines}; }
      $&;
   }
}

sub CLOSE { !@{$_[0]->lines} }

#######################################################################################

package Polymake::Core::CPlusPlus::No;
use Polymake::Struct (
   [ new => '$' ],
   [ '$application' => 'weak( #1 )' ],
);

sub end_loading { undef shift->application->cpp; }
sub embedded_rules_cnt { 0 }

sub AUTOLOAD {
   my ($self)=@_;
   generate_init_sources($self);
   bless $self, "Polymake::Core::CPlusPlus::perApplication";
   no strict 'refs';
   $AUTOLOAD=~s/::No(?=::)//;
   &$AUTOLOAD;
}

sub DESTROY { }

#######################################################################################
package Polymake::Core::CPlusPlus::perApplication;

use Polymake::Struct (
   [ new => '$' ],
   [ '$application' => 'weak( #1 )' ],
   '$so_name',
   '$so_timestamp',
   '$so_mirrored',
   '$functions_begin',
   '$functions_end',
   '$embedded_rules_begin',
   [ '$embedded_rules_cnt' => '0' ],
   '%lacking_types',		# PropertyType => 1  until a temporary shmod loaded
   '%all_lacking_types',	# %lacking_types accumulated since program start
   '%lacking_templates',
   '@lacking_auto_functions',
   '@obsolete_auto_functions',
   '$will_update_sources',
   '%extensions',		# Extension => [ "path", timestamp, so_mirrored ]
);

sub new {
   my $self=&_new;
   my $build_top= $self->application->installTop eq $InstallTop
                  ? $InstallArch
		  : $self->application->installTop."/build.$Arch";
   my $app_name=$self->application->name;
   $self->so_name="$build_top/lib/$app_name$dl_suffix";
   if (-e $self->so_name) {
      if (-w _) {
	 # developer mode
	 if ((stat _)[9] < $wrapper_updated{$app_name}) {
	    compile_shared_module($self->application, "$build_top/apps/$app_name");
	 }
	 $self->so_timestamp=load($self,$self->so_name);

      } else {
	 # end-user mode: maybe there is a mirror
	 load($self,$self->so_name);

	 my $mirror_top=$wrapper_mirrors{$self->application->installTop};
	 if (defined($mirror_top) && -d "$mirror_top/apps/$app_name") {
	    $build_top="$mirror_top/build.$Arch";
	    $self->so_name="$build_top/lib/$app_name$dl_suffix";
	    if (-e $self->so_name) {
	       if ((stat _)[9] < $wrapper_updated{$app_name}) {
		  compile_shared_module($self->application, "$build_top/apps/$app_name");
	       }
	    } else {
	       create_extension_build_dir($self,$mirror_top);
	       compile_shared_module($self->application, "$build_top/apps/$app_name");
	    }
	    $self->so_timestamp=load_mirror($self,$self->so_name);
	    $self->so_mirrored=1;
	 } else {
	    $self->so_mirrored=-1;
	 }
      }

   } elsif (has_source_files($self->application->top) && eval { generate_init_sources($self) }) {
      # this application has not yet been built for the current architecture
      compile_shared_module($self->application, "$build_top/apps/$app_name");
      if (-e $self->so_name) {	# in an early development phase, there might be no clients at all, hence this test
	 $self->so_timestamp=load($self,$self->so_name);
      }

   } else {
      bless $self, "Polymake::Core::CPlusPlus::No";
   }
   $self;
}

sub load {
   my ($self, $so_name)=@_;
   $root ||= new CPlusPlus;
   $self->functions_begin=@{$root->functions};
   $self->embedded_rules_begin=@{$root->embedded_rules};
   my $so_timestamp=load_shared_module($so_name);
   $self->functions_end=$#{$root->functions};
   $self->embedded_rules_cnt=@{$root->embedded_rules}-$self->embedded_rules_begin;
   $so_timestamp;
}

sub load_mirror {
   my ($self, $so_name)=@_;
   my $so_timestamp=load_shared_module($so_name);
   $self->functions_end=$#{$root->functions};
   $so_timestamp;
}

sub load_extension {
   my ($self, $extension)=@_;
   my $app_name=$self->application->name;
   my $ext_dir=$extension->dir;
   my $build_top=$ext_dir;
   $build_top =~ s|^$InstallTop|$InstallArch|o
   or $build_top.="/build.$Arch";
   my $so_name="$build_top/lib/$app_name$dl_suffix";
   if (-e $so_name) {
      if (-w _) {
	 # this is a private extension
	 if ((stat _)[9] < $wrapper_updated{"$ext_dir/$app_name"}) {
	    compile_shared_module($self->application, "$build_top/apps/$app_name");
	 }
	 $self->extensions->{$ext_dir}=[ $so_name, load($self,$so_name), 0 ];

      } else {
	 # this is an installed extension: maybe there is a mirror
	 load($self,$so_name);
	 my $mirror_top=$wrapper_mirrors{$ext_dir};
	 if (defined($mirror_top) && -d "$mirror_top/apps/$app_name") {
	    $build_top="$mirror_top/build.$Arch";
	    $so_name="$build_top/lib/$app_name$dl_suffix";
	    if (-e $so_name) {
	       if ((stat _)[9] < $wrapper_updated{"$ext_dir/$app_name"}) {
		  compile_shared_module($self->application, "$build_top/apps/$app_name");
	       }
	    } else {
	       create_extension_build_dir($self,$mirror_top);
	       compile_shared_module($self->application, "$build_top/apps/$app_name");
	    }
	    $self->extensions->{$ext_dir}=[ $so_name, load_mirror($self,$so_name), 1 ];
	 } else {
	    $self->extensions->{$ext_dir}=[ undef, 0, -1 ];
	 }
      }

   } elsif (has_source_files($extension->app_dir($self->application))) {
      # this extension has not yet been built for the current architecture
      create_extension_build_dir($self,$ext_dir);
      compile_shared_module($self->application, "$build_top/apps/$app_name");
      $self->extensions->{$ext_dir}=[ $so_name, load($self,$so_name) ];

   } else {
      # this extension has nothing to contribute to the current application
      return;
   }

   $so_name;
}

sub has_source_files {
   my $dir=shift;
   scalar(grep { not m{/(?:Makefile\.inc|polymake\.h)$} } glob("$dir/{src,include}/*"))
}

sub create_extension_build_dir {
   my ($self, $ext_dir, $primary)=@_;
   my $build_dir="$ext_dir/build.$Arch";
   if (!-d $build_dir) {
      if (! -f "$ext_dir/Makefile") {
	 open MF, ">$ext_dir/Makefile";
	 print MF <<".";
ProjectTop := $InstallTop
include \${ProjectTop}/support/extension.make
.
	 close MF;
      }
      if (-d "$ext_dir/.svn") {
	 my $ignore=`svn propget svn:ignore $ext_dir`;
	 if ($ignore !~ /^build\.\*$/m) {
	    my $tf=new Tempfile;
	    open my $IGN, ">$tf.prop";
	    print $IGN $ignore, "Makefile\nbuild.*\n";
	    close $IGN;
	    system("svn propset svn:ignore -F $tf.prop $ext_dir")
	    and warn_print( <<"." );
Adding 'Makefile' and 'build.*' to svn:ignore property of directory $ext_dir failed;
Please investigate the reason and perform it manually.
.
	 }
      }
   }

   my $dir="$build_dir/apps/".$self->application->name;
   foreach ($build_dir, "$build_dir/lib", "$build_dir/apps", $dir, "$dir/perl") {
      -d $_ or mkdir($_,0755) or die "Can't create directory $_: $!\n";
   }

   open MF, ">$dir/Makefile" or die "Can't create $dir/Makefile: $!/n";
   if ($primary) {
      if ($self->application->installTop ne $InstallTop) {
	 print MF "AppTop := $ext_dir\n";
      }
   } else {
      print MF "ExtensionTop := $ext_dir\n";
   }
   print MF <<".";
BuildDir := ../..
ProjectTop := $InstallTop
include $InstallArch/conf.make
include \${ProjectTop}/support/build.make
.
   close MF;
}

sub guess_file_location {
   my ($self, $src_file)=@_;
   if (index($src_file,$self->application->installTop)==0) {
      ($self->application->installTop, $self->so_name, $self->so_timestamp, $self->so_mirrored)
   } else {
      my $app_name=$self->application->name;
      if ($src_file =~ s{^(.*)(?=/apps/$app_name/(?:src|include)/)}{}) {
	 if (defined (my $ext=$self->extensions->{$1})) {
	    return ($1, @$ext);
         }
         if ($self->so_mirrored) {
            my $old_path=length($1);
            $src_file =~ s|/src/perl/wrap-|/src/|;
            if (-f $self->application->installTop.$src_file) {
               substr($_[1],0,$old_path)=$self->application->installTop;
               return ($self->application->installTop, $self->so_name, $self->so_timestamp, $self->so_mirrored)
            }
            while (my ($ext_top, $ext)=each %{$self->extensions}) {
               if (-f $ext_top.$src_file) {
                  substr($_[1],0,$old_path)=$ext_top;
                  return ($ext_top, @$ext);
               }
            }
         }
      }
      ()
   }
}

sub end_loading {
   my ($self)=@_;
   bind_functions($self,$self->functions_begin,$self->functions_end);
}

sub get_embedded_rules {
   my $self=shift;
   my $lines=$self->embedded_rules_cnt;
   $self->embedded_rules_cnt=0;
   my $handle=Symbol::gensym;
   select(select $handle);
   tie *$handle, Polymake::Core::CPlusPlus::EmbeddedRules, (splice @{$root->embedded_rules}, $self->embedded_rules_begin, $lines);
   $self->application->compile_scope->begin_locals;
   local_incr($from_embedded_rules);
   $self->application->compile_scope->end_locals;
   $handle
}

my %builtin2proxy=( int => 'NumProxy', double => 'NumProxy', 'std::string' => 'StringProxy', bool => 'BoolProxy' );

sub add_type {
   my ($self, $proto)=splice @_,0,2;
   my $opts=$proto->cppoptions=new Options(@_);
   if ($opts->special) {
      $opts->name ||= $opts->special || $proto->name;
      check_descr($proto);

   } elsif ($opts->builtin) {
      if (ref($opts->builtin)) {
	 $opts->name ||= $proto->name;
	 if (ref($opts->builtin) eq "Polymake::Enum") {
	    while (my ($key, $val)=each %{$opts->builtin}) {
	       my $ref=readonly_deep(bless \$val, $proto->pkg);
	       namespaces::export_sub($proto->application->pkg, define_function($proto->application->pkg, $key, sub { $ref }));
	    }
	 }
      } else {
	 $opts->name ||= $opts->builtin;
	 $root->builtins->{$opts->builtin}=$proto;
	 if (defined (my $proxy_class=$builtin2proxy{$opts->builtin})) {
	    no strict 'refs';
	    push @{$proto->pkg."::ISA"}, "Polymake::Core::CPlusPlus::$proxy_class";
	 }
      }
      check_descr($proto);

   } else {
      if (!is_code($opts->name)) {
	 if ($opts->name =~ /^(?: [\w:]+:: )?$/x) {
	    $opts->name .= $proto->name;
	 }
	 if (!@{$opts->include}) {
	    push @{$opts->include}, $opts->name =~ /^[A-Z]/ ? $opts->name.".h" : ($opts->name =~ /(?:^std::)?(\w+)$/)[0];
	 }
	 $opts->name .= $opts->append_template_params;
      }
      create_methods($self, $proto);
   }
}

sub add_type_template {
   my ($self, $super_pkg)=splice @_,0,2;
   my $opts=new Options(@_);
   if (!is_code($opts->name)) {
      if ($opts->name =~ /^(?: [\w:]+:: )?$/x) {
	 $opts->name .= ($super_pkg =~ /([^:]+)$/)[0];
      }
      if (!@{$opts->include} && $opts->template_params ne "*") {
	 push @{$opts->include}, $opts->name =~ /^[A-Z]/ ? $opts->name.".h" : ($opts->name =~ /(?:^std::)?(\w+)$/)[0];
      }
   }
   if (!exists $root->templates->{$super_pkg} && $opts->template_params ne "*") {
      $self->lacking_templates->{$super_pkg}=$opts;
      ensure_update_sources($self) unless $self->will_update_sources;
   }
   if ($opts->builtin) {
      if (is_code($opts->builtin)) {
	 if ($super_pkg =~ /::(array|map|pair)$/i) {
	    $root->builtins->{lc($1)} ||= $super_pkg;
	 }
      } else {
	 croak( "template type can't be declared as a C++ built-in type" );
      }
   } else {
      define_operators($self,$opts,$super_pkg);
   }
   define_function($super_pkg, "c++options",
		   sub {
		      if ((my $proto)=@_) {
			 my $clone=inherit_class([ @$opts ], $opts);
			 weak($clone->finalize_with=$proto);
			 if (is_code($opts->builtin)) {
			    $clone->builtin=$opts->builtin->($proto);
			 }
			 $clone
		      } else {
			 $opts
		      }
		   });
}

sub add_template_instance {
   my ($self, $proto, $cpp_opts_sub, $defer)=@_;
   $proto->cppoptions=$cpp_opts_sub->($proto);
   if (!$proto->abstract && $proto->cppoptions->template_params ne "*") {
      if ($proto->cppoptions->builtin) {
	 # this may be required when accessing to the C++ object via TryCanned
	 $proto->cppoptions->descr=$root->classes->{$proto->pkg} || sub { $proto->cppoptions->descr=check_descr($proto,1) };
	 define_operators($self,$proto->cppoptions,$proto->pkg,$proto);
      } elsif ($defer && !exists $root->classes->{$proto->pkg}) {
	 $proto->cppoptions->descr=sub { create_methods($self, $proto,$cpp_opts_sub->()) };
	 $proto->construct=sub : method {
	    my $proto=shift;  $proto->cppoptions->descr->();
	    $proto->construct->(@_);
	 };
	 $proto->parse_string=sub : method {
	    my $proto=shift;  $proto->cppoptions->descr->();
	    $proto->parse_string->(@_);
	 };
      } else {
	 create_methods($self, $proto,$cpp_opts_sub->());
      }
   }
}

sub add_auto_function {
   my ($self, $name, $signature, $attrs, $options)=@_;
   my ($pkg, $srcfile)=caller;
   my $application=$self->application;
   my $returns;
   my $flags=0;
   $flags |= $func_is_method if delete $attrs->{method};
   $flags |= $func_is_void if delete $attrs->{void};

   if (delete $attrs->{lvalue}) {
      if ($flags & $func_is_void) {
	 croak( "Attributes 'lvalue' and 'void' are mutually exclusive" );
      }
      $flags |= $func_is_lvalue;
   }
   if (delete $attrs->{lvalue_opt}) {
      if (!($flags & $func_is_method)) {
	 croak("Attribute 'lvalue_opt' is only applicable to methods");
      }
      if ($flags & $func_is_void) {
	 croak( "Attributes 'lvalue_opt' and 'void' are mutually exclusive" );
      }
      $flags |= $func_is_lvalue_opt;
   }
   if (delete $attrs->{non_const}) {
      if (!($flags & $func_is_method)) {
	 croak("Attribute 'non_const' is only applicable to methods");
      }
      if ($flags & ($func_is_lvalue|$func_is_lvalue_opt)) {
	 croak( "Attributes 'non_const', 'lvalue', and 'lvalue_opt' are mutually exclusive" );
      }
      $flags |= $func_is_non_const;
   }
   if (defined ($returns=delete $attrs->{returns})) {
      if ($flags & $func_is_void) {
	 croak( "Attributes 'returns' and 'void' are mutually exclusive" );
      }
      $options->{macro_call}="Pkg( ";
      $flags |= $func_has_prescribed_ret_type;
   }
   my @kw;
   if (defined (my $kw=delete $attrs->{keywords})) {
      @kw=(keywords=>$kw);
   }
   my $subst_const_op=delete $attrs->{subst_const_op};

   if (keys %$attrs) {
      croak("Unknown attribute", keys(%$attrs)>1 && "s", " for a C++ function: ", join(", ", keys %$attrs));
   }

   my $auto_func= ($flags & $func_is_method and $name eq "construct")
                  ? new Constructor()
		  : new AutoFunction($name, $flags, $application, $options);
   my $code;
   if (defined($auto_func->embedded)) {
      $auto_func->name ||= $name;
      if (defined $signature) {
	 $auto_func->flags |= $func_is_ellipsis if $signature =~ /\@\s*$/;
	 $signature =~ s/($type_re)&/$1/go;
      }
      $auto_func->template_params=0;
      my $descr=$root->regular_functions->[$auto_func->embedded];
      $auto_func->embedded=$srcfile;
      $descr->auto_func=$auto_func;
      undef $auto_func->inst_cache;
      $code= defined($returns) ? sub { unshift @_, $returns; &{resolve_regular_function($auto_func, $descr, \@_)} }
                               : sub { &{resolve_regular_function($auto_func, $descr, \@_)} };

   } else {
      croak( "C++ function template without signature" ) unless defined $signature;
      if ($from_embedded_rules) {
	 $auto_func->embedded=$srcfile;
	 $signature =~ s/($type_re)&/lvalue($1)/go;
      }
      $signature=$auto_func->prepare($signature, $pkg, $application);

      if (!($flags & $func_is_method) && $pkg ne $application->pkg) {
	 my $app=$application->pkg;
	 (my $pkg_prefix=$pkg)=~s/^$app\:://;
	 $pkg_prefix=~tr/:/_/;
	 substr($auto_func->wrapper_name,0,0)=$pkg_prefix."__";
      }
      check_twins($auto_func);

      if ($flags & $func_is_method and $name eq 'operator') {
	 my $obj;
	 my $closure=sub { unshift @_, $obj;  &{resolve_auto_function($auto_func, \@_)} };
	 $code=sub { $obj=shift; $closure };
	 declare_lvalue($closure, 1) if $flags & ($func_is_lvalue|$func_is_lvalue_opt);
      } else {
	 $code= defined($returns) ? sub { unshift @_, $returns; &{resolve_auto_function($auto_func, \@_)} }
	                          : sub { &{resolve_auto_function($auto_func, \@_)} };
      }
      set_method($code) if $flags & $func_is_method;
      declare_lvalue($code, 1) if $flags & ($func_is_lvalue|$func_is_lvalue_opt);
   }

   if (@kw) {
      $auto_func->opt_hashes=@{$kw[1]};
   }

   if (defined $signature) {
      if ($name eq "operator") {
	 overload::OVERLOAD($pkg, '&{}' => $code);
	 ()
      } else {
	 if ($name eq "construct") {
	    if (defined $subst_const_op) {
	       no strict 'refs';
	       namespaces::subst_const_op($application->pkg, $subst_const_op, sub { $pkg->new(@_) });
	    }
	 } else {
	    if (defined $subst_const_op) {
	       push @kw, subst_const_op => [ $pkg, $subst_const_op ];
	    }
	    if (@{$auto_func->extra_template_params}) {
	       push @kw, extra_template_params => 1;
	       if (@{$auto_func->args_for_deduction}) {
		  push @kw, wrapper => [ undef, sub { unshift_undef_args(1); &$code; } ];
	       }
	    }
	 }
	 if (@{$auto_func->args_for_deduction} && !@{$auto_func->extra_template_params}) {
	    push @kw, wrapper => sub { Polymake::Overload::check_matching_template_params($auto_func->args_for_deduction,
											  \@_, \&guess_builtin_type);
				       &$code; };
	 }
	 ( $name, $signature, $code, @kw )
      }
   } else {
      define_function($pkg, $name, $code);
      ()
   }
}

#######################################################################################
package Polymake::Core::CPlusPlus;

declare ($from_embedded_rules, $root);
declare (@to_mangled_defconstr, $nm_util, $cppfilt_util, $MAKEFLAGS, %wrapper_mirrors, %wrapper_updated);

declare $constructor_re=qr{::$id_re (?:\[.*?\])? \(\)$}xo;
declare $wrapped_constructor_re=qr{^PolymakeTestWrap<(.*)>::PolymakeTestWrap (?:\[.*?\])? \(\)$}xo;

my $custom_handler=$Custom->app_handler(undef,__PACKAGE__);

$custom_handler->add('@to_mangled_defconstr', <<'.', $Customize::state_config);
how to derive the default constructor mangled name from the class typeinfo
.
$custom_handler->add('$nm_util', <<'.', $Customize::state_config);
path and optional flags to the nm utility
.
$custom_handler->add('$cppfilt_util', <<'.', $Customize::state_config);
path and optional flags to the c++filt utility
.
$custom_handler->add('$MAKEFLAGS', <<'.', $Customize::state_config);
options for make utility (especially -jN for parallel builds on multi-core machines)
.
$custom_handler->add('%wrapper_mirrors', <<'.', $Customize::state_hidden);
locations of the automatically generated source code of C++/perl wrappers
for read-only standard installation and/or extensions
.
$custom_handler->add('%wrapper_updated', <<'.', $Customize::state_hidden);
timestamps of last changes made to the automatically generated source code of C++/perl wrappers
.
if (!@to_mangled_defconstr) {
   require Polymake::Core::CPlusPlus_config;
   local $Customize::configure_state=$Customize::state_config|$Customize::state_saved;
   study_name_mangling();
   allow_parallel_make();
}

$custom_handler->cleanup;

# auto-function flags:
use Polymake::Enum qw( func: is_method=1 is_lvalue=2 is_lvalue_opt=4 is_non_const=8 is_void=16
		             is_ellipsis=32 is_top_object_method=64 has_prescribed_ret_type=128 );

# class description kind:
use Polymake::Enum qw( class_is: scalar container composite assoc_container=0x100 sparse_container=0x200 );

use Polymake::Struct (
   '@functions',
   '@regular_functions',
   '@embedded_rules',
   '%classes',
   '%builtins',
   '%templates',
   '%typeids',
   '%auto_functions',
   '$auto_default_constructor',
   '$auto_convert_constructor',
   '$auto_assignment',
   '$auto_conversion',
   '$indirect_wrapper',
   '@auto_assoc_methods',
   '@auto_set_methods',
);

use Polymake::Enum qw( assoc: helem find exists delete_void delete_ret );

use Polymake::Ext;

sub new {
   my $self=&_new;

   $Object_pseudo_proto=_new PropertyType("Object", "Polymake::Core::Object", undef);
   $Object_pseudo_options=$Object_pseudo_proto->cppoptions=new ObjectOptions(name=>"Object", builtin=>"perl::Object");
   define_function("Polymake::Core::Object", "c++options", \&ObjectType::cppoptions);

   load_shared_module($corelib);

   # create the standard constructors right now
   my $dc=new Constructor;
   $dc->prepare("");
   $dc->name="";
   $self->auto_default_constructor=$self->auto_functions->{$dc->wrapper_name}=$dc;

   my $cc=$self->auto_convert_constructor=new Constructor;
   $cc->prepare("*");
   $cc->name="";
   $self->auto_convert_constructor=$self->auto_functions->{$cc->wrapper_name}=$cc;

   $self->auto_assignment=$self->auto_functions->{"=ass"}=new SpecialOperator("assign");
   $self->auto_conversion=$self->auto_functions->{".cnv"}=new SpecialOperator("convert");
   $self->indirect_wrapper=$self->auto_functions->{".wrp"}=new IndirectWrapper;
   create_assoc_methods($self);

   $self;
}

sub check_descr {
   my ($proto, $need_now)=@_;
   $root->classes->{$proto->pkg}
   or do {
      defined($proto->cppoptions) or croak( "type ", $proto->full_name, " has no C++ binding" );
      my $self=($proto->covering_app || croak( "don't know where to place the definition of C++ type ", $proto->full_name ))->cpp;
      $self->lacking_types->{$proto}=1;
      $self->all_lacking_types->{$proto}=1;
      if ($need_now) {
	 compile_load_temp_shared_module($self);
	 ensure_update_sources($self) unless $self->will_update_sources;
	 $root->classes->{$proto->pkg}
      } else {
	 undef;
      }
   }
}

*Polymake::Core::PropertyType::check_descr=\&check_descr;

sub Polymake::Core::PropertyType::get_cpp_representation {
   my $proto=shift;
   check_descr($proto,1)->get_cpp_representation($proto);
}

sub get_magic_cpp_class {
   my $typeid=get_magic_typeid(@_);
   $typeid && do {
      if (defined (my $descr=$root->typeids->{$typeid})) {
	 scalar($descr->get_cpp_representation);
      } else {
	 demangle($typeid);
      }
   }
}

sub Polymake::Core::PropertyType::get_element_type {
   my $proto=shift;
   if ($proto->dimension==1) {
      if ($proto->cppoptions && !$proto->cppoptions->builtin) {
	 get_type_proto(check_descr($proto,1)->vtbl, 1)
      } else {
	 $proto->param;
      }
   } else {
      undef
   }
}

sub Polymake::Core::PropertyType::guess_element_type {
   my $data=shift;
   if (is_object($data) && UNIVERSAL::can($data, "type") &&
       defined (my $type=$data->type->get_element_type)) {
      return $type;
   }
   my $first=$data->[0];
   if (is_object($first) && UNIVERSAL::can($first, "type")) {
      $first->type;
   } elsif (is_integer($first)) {
      $root->builtins->{int};
   } elsif (is_float($first)) {
      $root->builtins->{double};
   } else {
      $root->builtins->{"std::string"};
   }
}

sub guess_builtin_type {
   if (my $class=ref($_[0])) {
      if ($class eq "ARRAY") {
	 # deliberately interpret an empty array as an empty set of indices
	 my $elem_proto= @{$_[0]} ? guess_builtin_type($_[0]->[0]) : $root->builtins->{int};
	 return $root->builtins->{array}->generic_type($elem_proto);
      }
      ### FIXME: mapping for HASH
      if (&is_object) {
	 if (defined (my $proto=UNIVERSAL::can($_[0],"prototype"))) {
	    $proto=$proto->();
	    if ($proto->cppoptions && $proto->cppoptions->builtin) {
	       return $proto;
	    }
	 }
      }
   } else {
      return $root->builtins->{bool} if &is_boolean;
      return $root->builtins->{int} if &is_integer;
      return $root->builtins->{double} if &is_float;
      return $root->builtins->{"std::string"} if &is_string;
   }
   croak( "don't know how to map ", ref($_[0]) || "'$_[0]'", " to a C++ type" );
}

# private:
sub set_embedded {
   my ($self, $embedded)=@_;
   if (defined($self->embedded)) {
      if ($self->embedded ne $embedded) {
	 croak( $self->complain_source_conflict, ": conflicting files $embedded and ", $self->embedded );
      }
   } else {
      $self->embedded=$embedded;
   }
}

*set_application=\&PropertyParamedType::set_application;
*set_extension=\&PropertyParamedType::set_extension;

#######################################################################################

sub resolve_auto_function {
   my ($auto_func, $args)=@_;
   $auto_func->find_instance($args) || do {
      my $lacking=new LackingInstance(@_);
      my $self=$lacking->application->cpp;
      compile_load_temp_shared_module($self,$lacking);
      push @{$self->lacking_auto_functions}, $lacking;
      ensure_update_sources($self) unless $self->will_update_sources;
      $auto_func->find_instance($args)
         or die( "an instance of C++ ", instanceof Operator($auto_func) ? ("operator ", $auto_func->sign)
		 : ("function ", $auto_func->perl_name), " was not created as expected\n" );
   }
}

sub resolve_regular_function {
   my ($auto_func, $descr, $args)=@_;
   my $indirect_wrapper=$auto_func->inst_cache ||= do {
      my $code=create_function_wrapper($descr, $auto_func->application->pkg);
      (my $void_flag, @{$auto_func->lvalue_params})=@{$code->()};
      $auto_func->flags |= $func_is_void if $void_flag;
      my $indirect_descr=$root->indirect_wrapper->inst_cache->{$descr->arg_types} || do {
	 my $self=$auto_func->application->cpp;
	 my $lacking=new LackingRegular($auto_func,$descr);
	 compile_load_temp_shared_module($self,$lacking);
	 push @{$self->lacking_auto_functions}, $lacking;
	 ensure_update_sources($self) unless $self->will_update_sources;
	 $root->indirect_wrapper->inst_cache->{$descr->arg_types}
	    or die( "the wrapper for C++ function ", $auto_func->perl_name, " was not created as expected\n" );
      };
      my $n_args=@{$descr->arg_types};
      ++$n_args if $auto_func->flags & $func_has_prescribed_ret_type;
      set_number_of_args($code, $n_args, $auto_func->opt_hashes, $auto_func->flags & $func_is_ellipsis, $indirect_descr->wrapper);
      $code
   };
   $auto_func->check_lvalue_args($args);
   $indirect_wrapper
}

sub bind_functions {
   my ($self, $begin, $end, $subst_for_temp_src)=@_;

   foreach my $descr (@{$root->functions}[$begin..$end]) {
      my $auto_func=$root->auto_functions->{$descr->name};
      my ($src_file, $bunch);
      if (ref($auto_func) eq "HASH") {
	 $bunch=$auto_func;
	 if (defined $subst_for_temp_src) {
	    $src_file=$subst_for_temp_src->{$descr->source_file};
	 } else {
	    ($src_file=$descr->source_file) =~ s|/src/perl/wrap-|/src/|;
	 }
	 $auto_func= $bunch->{$src_file} || $bunch->{$self->application->name};
      }

      unless (defined $auto_func) {
	 my $delete_func=(defined($bunch) ? $bunch->{$src_file} : $root->auto_functions->{$descr->name})=
            new AutoFunction($descr->name,0,undef);
	 $delete_func->template_params=-1;
	 $delete_func->seen_in->{$descr->source_file}=2;
	 push @{$self->obsolete_auto_functions}, $delete_func;
	 ensure_update_sources($self) unless $self->will_update_sources;
	 dbg_print( "C++ function ", $descr->name, " defined in ", $descr->source_file,
		    " has no binding declared in the rules, treating as obsolete" ) if $Verbose::cpp;
	 next;
      }

      if (ref($auto_func->template_params)) {
	 $auto_func->seen_in->{$descr->source_file}=1;
	 $descr->auto_func=$auto_func;			# help guessing the right set of headers for non-persistent return types

	 my $sub;
	 if (defined $descr->wrapper) {
	    $sub=create_function_wrapper($descr, $self->application->pkg);
	    $auto_func->prepare_wrapper_code($sub);
	 } else {
	    $sub=$auto_func->create_fallback($descr);
	 }

	 my $last=$#{$descr->arg_types};
	 my ($inst_cache, $extra);
	 if ($last>=0) {
	    $inst_cache=$auto_func->inst_cache;
	    $extra=@{$auto_func->extra_template_params};
	    for (my $tp=0; $tp<$last; ++$tp) {
	       if ($tp>=$extra && $auto_func->template_params->[$tp-$extra]->[1] == $func_is_lvalue_opt) {
		  $inst_cache=($inst_cache->{$descr->arg_types->[$tp]}->[$descr->arg_types->[$tp]] ||= { });
	       } else {
		  $inst_cache=($inst_cache->{$descr->arg_types->[$tp]} ||= { });
	       }
	    }
	 }
	 (( $last>=0
	    ? $last>=$extra && $last-$extra <= $#{$auto_func->template_params} && $auto_func->template_params->[$last-$extra]->[1] == $func_is_lvalue_opt
	      ? $inst_cache->{$descr->arg_types->[$last]}->[$descr->arg_types->[$last]]
	      : $inst_cache->{$descr->arg_types->[$last]}
	    : $auto_func->inst_cache )
	  &&= (defined($subst_for_temp_src)
	       ? next
	       : die "multiple instances of function template ", $auto_func->perl_name, "(",
		     join(", ", map { demangle($_) || "UNDECIPHERABLE( $_ )" } @{$descr->arg_types}),
	             ")\n")
	 ) =$sub;

      } elsif ($auto_func->template_params==0) {
	 ($auto_func->inst_cache->{$descr->arg_types} &&= next) =$descr;	# repetition can occur when mixing several applications together

      } elsif (!$auto_func->seen_in->{$descr->source_file}) {
	 $auto_func->seen_in->{$descr->source_file}=2;
	 dbg_print( "C++ function ", $descr->name, " defined in ", $descr->source_file,
		    " has no binding declared in the rules, treating as obsolete" ) if $Verbose::cpp;
      }
   }
}

sub try_merge_auto_functions {
   my ($f1, $f2)=@_;
   return 0 if $f1->embedded ne $f2->embedded;
   return 1 if $f1->application==$f2->application;
   if ($f1->name ne "construct" && defined (my $common_app=$f1->application->common($f2->application))) {
      $f2->application=$common_app;
      return 1;
   }
   0
}

sub check_twins {
   my ($auto_func)=@_;
   if (defined (my $twin=$root->auto_functions->{$auto_func->wrapper_name})) {
      if (ref($twin) eq "HASH") {
	 foreach my $other (values %$twin) {
	    if (try_merge_auto_functions($auto_func,$other)) {
	       push @{$other->include}, @{$auto_func->include};
	       $_[0]=$other;
	       return;
	    }
	 }
	 $twin->{$auto_func->embedded || $auto_func->application->name}=$auto_func;
      } elsif (try_merge_auto_functions($auto_func,$twin)) {
	 push @{$twin->include}, @{$auto_func->include};
	 $_[0]=$twin;
      } else {
	 $root->auto_functions->{$auto_func->wrapper_name}=
	    { map { ($_->embedded || $_->application->name, $_) } $auto_func, $twin };
      }
   } else {
      $root->auto_functions->{$auto_func->wrapper_name}=$auto_func;
   }
}

#######################################################################################

my %op_descr=( '+' => [ ':add', 1 ], '-' => [ ':sub', 1 ], '*' => [ ':mul', 1 ], '/' => [ ':div', 1 ], '%' => [ ':mod', 1 ],
	       '<<' => [ ':lsh', 1 ], '>>' => [ ':rsh', 1 ], '&' => [ ':and', 1 ], '|' => [ ':_or', 1 ], '^' => [ ':xor', 1 ],
	       '+=' => [ '=add', 1+$func_is_lvalue ], '-=' => [ '=sub', 1+$func_is_lvalue ], '*=' => [ '=mul', 1+$func_is_lvalue ],
	       '/=' => [ '=div', 1+$func_is_lvalue ], '%=' => [ '=mod', 1+$func_is_lvalue ], '<<=' => [ '=lsh', 1+$func_is_lvalue ],
	       '>>=' => [ '=rsh', 1+$func_is_lvalue ], '&=' => [ '=and', 1+$func_is_lvalue ], '|=' => [ '=_or', 1+$func_is_lvalue ],
	       '^=' => [ '=xor', 1+$func_is_lvalue ],
	       'neg' => [ '.neg', 0 ], '!' => [ '.not', 0 ], '~' => [ '.com', 0 ], 'bool' => [ '.boo', 0 ],
	       '++' => [ '.inc', $func_is_lvalue ], '--' => [ '.dec', $func_is_lvalue ],
	       '==' => [ ':_eq', 1 ], '!=' => [ ':_ne', 1 ], '<' => [ ':_lt', 1 ],
	       '<=' => [ ':_le', 1 ], '>' => [ ':_gt', 1 ], '>=' => [ ':_ge', 1 ],
	       '<=>' => [ 'cmp', 1 ], '**' => [ 'pow', 1 ], 'abs' => [ 'abs', 0 ],
	     );

my %op_groups=( arith => [ qw( neg ! + - * / += -= *= /= ) ],
		arith_nofloat => [ qw( neg ! +F -F *F /F +=F -=F *=F /=F ) ],
		sets => [ qw( ~ + += - -= * *= ^ ^= ) ],
		bits => [ qw( ~ & | ^ &= |= ^= ) ],
		compare => [ qw( == != < <= > >= ) ],
		string => [ ],					# just to enable the default string ops in define_operators
	      );

my @assoc_descr=(
   [ $assoc_helem,       ":brk",          "[]",           1+$func_is_lvalue_opt ],
   [ $assoc_find,        "assoc_find",    "*,*",          0,
     name => "pm::perl::find_element", include => [ "perl/assoc.h" ] ],
   [ $assoc_exists,      "exists",        "*",            $func_is_method ],
   [ $assoc_delete_void, "erase",         "*",            $func_is_method+$func_is_non_const+$func_is_void ],
   [ $assoc_delete_ret,  "assoc_delete",  "lvalue(*),*",  0,
     name => "pm::perl::delayed_erase", include => [ "perl/assoc.h" ], macro_call => "Tmp( " ],
);

sub create_assoc_methods {
   my ($self)=@_;
   $#{$self->auto_assoc_methods}=$#{$self->auto_set_methods}=$#assoc_descr;
   foreach my $descr (@assoc_descr) {
      my ($i, $name, $signature, $flags, @options)=@$descr;
      if ($name =~ /^\w/) {
	 my $auto_func=new AutoFunction($name, $flags, undef, @options);
	 $auto_func->prepare($signature);
	 $self->auto_functions->{$auto_func->wrapper_name}=$auto_func;
	 my $code=$self->auto_assoc_methods->[$i]=sub { &{resolve_auto_function($auto_func, \@_)} };
	 set_method($code) if $flags & $func_is_method;
	 declare_lvalue($code, 1) if $flags & ($func_is_lvalue|$func_is_lvalue_opt);
      } else {
	 my $op=new Operator($name, $flags, $signature);
	 $self->auto_functions->{$name}=$op;
	 $self->auto_assoc_methods->[$i]=$op->code;
      }
   }
   $self->auto_set_methods->[$assoc_helem]=\&no_set_method;
   $self->auto_set_methods->[$assoc_find]=\&no_set_method;
   $self->auto_set_methods->[$assoc_exists]=$self->auto_assoc_methods->[$assoc_exists];
   $self->auto_set_methods->[$assoc_delete_void]=$self->auto_assoc_methods->[$assoc_delete_void];
   $self->auto_set_methods->[$assoc_delete_ret]=\&no_set_method;
}

sub no_set_method { croak("This operation is not defined on sets") }

sub add_operator {
   my ($self, $sign, $pkg)=@_;
   my ($op, $code);
   if ($sign =~ s/^ ([^\s(]+) \s*\(\s* ($qual_id_re) (?: \s*\( ($balanced_re) \) )? \s*\)\s*$/$1/xo) {
      my ($cppname, $signature)=($2,$3);
      my $descr=$op_descr{$sign} or croak( "operator $sign can't have C++ binding" );

      # if the function is declared with one argument less than expected, it must be a method,
      # otherwise a free function with an appropriate number of arguments is assumed
      my $flags=0;
      if (defined $signature) {
         $flags=$func_is_method if $signature =~ ($descr->[1] ? qr/^\s* \* \s*$/x : qr/^\s*$/);
      } else {
         $signature= $descr->[1] ? "*,*" : "*";
      }
      $op=new AutoFunction($descr->[0], $flags, $self->application);
      $op->prepare($signature, $pkg, undef);
      check_twins($op);

      if ($sign eq '<=>') {
	 $code=sub {
	    if (pop) {
	       - &{ resolve_auto_function($op,\@_) }
	    } else {
	       &{ resolve_auto_function($op,\@_) }
	    }
	 }
      } elsif ($sign eq '**') {
         $code=sub {
            if (pop) {
               missing_op(@_,1,'**');
            } else {
               &{ resolve_auto_function($op,\@_) }
	    }
	 }
      } else {
         $code=sub { splice @_,-2; &{ resolve_auto_function($op,\@_) } }
      }

      $op->name=$cppname;
   } else {
      $sign =~ s/([A-Z])$//;
      my $special_flag=$1;
      my $descr=$op_descr{$sign} or croak( "operator $sign can't have C++ binding" );
      if ($descr->[0] !~ /^[.:=]/) {
         croak( "operator $sign must be matched to a C++ function or method" );
      }
      $code=($op=($root->auto_functions->{$descr->[0]} ||= new Operator(@$descr, $sign, $special_flag)))->code;
   }

   ($op->flags & $func_is_lvalue) && !$_[3]++ ? ($sign => $code, '=' => \&overload_clone_op) : ($sign => $code);
}

sub define_operators {
   my ($self, $opts, $pkg, $proto)=@_;
   if (length($opts->operators)) {
      my $clone_added;
      overload::OVERLOAD($pkg,
			 fallback => 0,
			 nomethod => \&missing_op,
			 '""' => \&convert_to_string,
			 @PropertyType::string_ops,
			 map { add_operator($self,$_,$pkg,$clone_added) }
			 map { /^\@/ ? @{$op_groups{$'} or croak( "unknown operator group $'" )} : $_ }
			    ($opts->operators =~ /[^\s\(]+ (?:\s*\( $balanced_re \))?/gxo)
			);
   }
   if ($opts->fields) {
      if ($proto) {
	 PropertyParamedType::set_field_names($proto, @{$opts->fields});
      } else {
	 my $i=0;
	 foreach my $field_name (@{$opts->fields}) {
	    define_function($pkg, $field_name, Struct::create_accessor($i++, \&composite_access));
	 }
      }
   }
}

#######################################################################################

sub construct_any : method {
   assign_array_to_cpp_object((shift)->construct->(),@_,$PropertyType::trusted_value);
}

sub assign_any : method {
   if ($#_==1 && (ref($_[1]) eq "ARRAY" || UNIVERSAL::isa($_[1], ref($_[0])))) {
      assign_to_cpp_object(@_,$PropertyType::trusted_value);
   } else {
      assign_array_to_cpp_object(@_,$PropertyType::trusted_value);
   }
}

sub construct_parsed : method {
   my $proto=shift;
   if ($_[0] =~ /\S/) {
      my $obj=eval { assign_to_cpp_object($proto->construct->(),$_[0],$PropertyType::trusted_value) };
      if (!defined($obj)) {
	 if ($@ =~ /^(\d+)\t/) {
	    local $_=$_[0];
	    pos($_)=$1;
	    $proto->parse_error;
	 } else {
	    die $@;
	 }
      }
      $obj;
   } else {
      $proto->construct->();
   }
}

sub construct_composite_cpp_value : method {
   my ($proto)=@_;
   if (@_>2 && !($#_%2)) {
      my ($i,$last)=(1,$#_-1);
      for (; $i<=$last; $i+=2) {
	 if (!is_keyword($_[$i])) {
	    return &construct_any;
	 }
      }
      my $x=$proto->construct->();
      for ($i=1; $i<=$last; $i+=2) {
	 my $field=$_[$i];
	 $x->$field=$_[$i+1];
      }
      $x;
   } else {
      &construct_any;
   }
}

sub std_default_constructor : method {
   &{resolve_auto_function($root->auto_default_constructor, \@_)}
}

sub std_parsing_constructor : method {
   assign_to_cpp_object($_[0]->construct->(),$_[1],$PropertyType::trusted_value);
}

sub std_convert_constructor : method {
   if (ref($_[1]) eq "ARRAY") {
      &std_parsing_constructor;
   } else {
      &{resolve_auto_function($root->auto_convert_constructor, \@_)};
   }
}

sub no_default_constructor : method {
   croak( (@_>1 ? "No matching constructor for " : "No default constructor for "), $_[0]->full_name );
}

Struct::pass_original_object($_) for (\&construct_parsed, \&no_default_constructor);

sub define_constructors {
   my ($proto, $pkg, $symtab)=@_;
   if (defined($symtab) ||
       do { $symtab=get_pkg($pkg); no strict 'refs'; !${"$pkg\::construct"}++ }) {
      my $opts=$proto->cppoptions;
      if ($opts->default_constructor) {
	 Overload::_add($pkg, "construct", "", \&std_default_constructor);
	 Overload::_add($pkg, "construct", "\$", \&std_convert_constructor);
	 Overload::_add($pkg, "construct", "Polymake::_Sparse", \&std_parsing_constructor);
	 Overload::add_fallback($pkg, "construct",
				$opts->descr->kind & $class_is_composite ? \&construct_composite_cpp_value : \&construct_any);
      } else {
	 Overload::add_fallback($pkg, "construct", \&no_default_constructor);
      }
   }
   $proto->construct=Struct::pass_original_object(\&{*{$symtab->{construct}}});
}

sub can_inherit_constructors {
   my ($proto)=@_;
   defined($proto) && $proto->cppoptions && !$proto->cppoptions->builtin && !is_code($proto->cppoptions->descr);
}

sub create_methods {
   my ($self, $proto, $generic_opts)=@_;
   my $descr=check_descr($proto,1);
   if (defined($descr->generated_by)) {
      Carp::confess( "unexpected call of create_methods" );
   }
   if ($proto->cppoptions->builtin) {
      Carp::confess( "unexpected call of create_methods" );
   }
   # this is only called for types declared in perl (i.e. "persistent types"),
   # the relation between PropertyType and C++ class descriptors is unambigous
   $proto->dimension=obj_dimension($descr->vtbl);
   my $opts=$proto->cppoptions;
   $opts->descr=$descr;

   $proto->toString=\&convert_to_string;
   $proto->parse_string= $opts->default_constructor ? \&construct_parsed : \&no_default_constructor;
   my $symtab=get_pkg($proto->pkg);

   if ($descr->kind==$class_is_scalar) {
      if (!length($proto->cppoptions->operators)) {
	 overload::OVERLOAD($proto->pkg,
			    fallback => 0,
			    nomethod => \&missing_op,
			    '""' => \&convert_to_string,
			    @PropertyType::string_ops,
			   );
      }
      if ($descr->is_numeric) {
	 Overload::_add($root->builtins->{int}->pkg,    "construct", $proto->pkg, \&convert_to_int);
	 Overload::_add($root->builtins->{double}->pkg, "construct", $proto->pkg, \&convert_to_float);
      }

   } elsif ($descr->kind & $class_is_container) {
      if ($descr->kind & $class_is_assoc_container) {
	 $proto->toXML=\&assoc_container_toXML;
	 no strict 'refs';
	 push @{$proto->pkg."::ISA"}, "Polymake::Core::CPlusPlus::TiedHash";
      } else {
	 $proto->toXML=\&container_toXML;
	 no strict 'refs';
	 push @{$proto->pkg."::ISA"}, "Polymake::Core::CPlusPlus::TiedArray";
      }

   } elsif ($descr->kind & $class_is_composite) {
      if ($proto->cppoptions->fields) {
	 if (get_type_proto($descr->vtbl, 3) != scalar(@{$proto->cppoptions->fields})) {
	    croak( "number of field names does not match the C++ description of class ", $proto->full_name );
	 }
      }
      $proto->toXML=\&composite_toXML;
      no strict 'refs';
      push @{$proto->pkg."::ISA"}, "Polymake::Core::CPlusPlus::TiedCompositeArray";
   }

   if (exists $symtab->{construct}) {
      # already have at least one special constructor
      define_constructors($proto, $proto->pkg, $symtab);

   } elsif ($generic_opts) {
      if ($generic_opts->builtin) {
	 # each instance must have its own construction method
	 define_constructors($proto, $proto->pkg, $symtab);

      } elsif (!can_inherit_constructors($proto->super)) {
	 # no base class to inherit constructors from: define them in the generic SUPER-package
         define_constructors($proto, *{$symtab->{ISA}}->[0]);
      }
   } elsif (!can_inherit_constructors($proto->super)) {
      # no base class to inherit constructors from
      define_constructors($proto, $proto->pkg, $symtab);
   }

   if (!defined($generic_opts) || is_code($generic_opts->builtin)) {
      # operators for never-builtin parameterized types are created earlier: in add_type_template
      define_operators($self, $proto->cppoptions, $proto->pkg);
   }
}

# defer the choice of the right method until the first real usage,
# when all prototypes are created
sub container_toXML : method {
   my $proto=shift;
   my $descr=$proto->cppoptions->descr;
   $proto->toXML= do {
      if (defined (my $val_proto=get_type_proto($descr->vtbl, 1))) {
	 if ($descr->kind & $class_is_sparse_container) {
	    if (defined (my $elem_proto=get_type_proto($descr->vtbl, 0))) {
	       my $own_dim=$proto->dimension-$elem_proto->dimension;
	       if ($own_dim==1) {
		  sub { PropertyType::sparseArray_toXML($elem_proto, @_) };
	       } elsif ($own_dim==2) {
		  sub { PropertyType::sparseMatrix_toXML($val_proto, @_) };
	       } else {
		  undef;
	       }
	    } else {
	       undef;
	    }
	 } else {
	    defined($val_proto->toXML)
	    ? sub { PropertyType::nontrivialArray_toXML($val_proto, @_) }
	    : sub { PropertyType::trivialArray_toXML($val_proto, @_) };
	 }
      } else {
	 undef;
      }
   };
   $proto->toXML->(@_);
}

sub assoc_container_toXML : method {
   my $proto=shift;
   my $descr=$proto->cppoptions->descr;
   $proto->toXML= do {
      my @kv_proto=map { get_type_proto($descr->vtbl, $_) } 0,1;
      if (defined($kv_proto[0]) && defined($kv_proto[1])) {
	 my $pair_proto=$root->builtins->{pair}->generic_type(@kv_proto);
	 sub { PropertyType::assocContainer_toXML($pair_proto, @_) }
      } else {
	 undef
      }
   };
   $proto->toXML->(@_);
}

sub composite_toXML : method {
   my $proto=shift;
   my $descr=$proto->cppoptions->descr;
   $proto->toXML= do {
      my $elem_protos=get_type_proto($descr->vtbl, 2);
      my $trivial=1;
      foreach my $elem_proto (@$elem_protos) {
	 if (defined $elem_proto) {
	    if (defined($elem_proto->toXML)) {
	       $trivial=0;
	    }
	 } else {
	    undef $trivial; last;
	 }
      }
      if (defined $trivial) {
	 $trivial
	 ? \&PropertyType::trivialComposite_toXML
	 : sub { PropertyType::nontrivialComposite_toXML($elem_protos, @_) };
      } else {
	 undef;
      }
   };
   $proto->toXML->(@_);
}

sub dump_arg {
   if (&is_object) {
      if (my $proto=UNIVERSAL::can($_[0],"prototype")) {
	 $proto->()->full_name;
      } else {
	 ref($_[0]);
      }
   } else {
      ref($_[0]) || "'$_[0]'"
   }
}

sub missing_op {
   @_==4
   ? $_[2]
     ? croak( "undefined operator ", dump_arg($_[1]), " $_[3] ", dump_arg($_[0]) )
     : croak( "undefined operator ", dump_arg($_[0]), " $_[3] ", dump_arg($_[1]) )
   : croak( "undefined operator $_[2] ", dump_arg($_[0]) );
}

#######################################################################################

sub compile_shared_module {
   my ($app, $build_dir)=@_;
   my $errfile=new Tempfile;
   my $debug_flag= $debug && "Debug=y";
   warn_print( "Recompiling shared module for application ", $app->name, ", please be patient..." );
   my $rc=system("make -C $build_dir all $debug_flag $MAKEFLAGS" . (!$Verbose::cpp && ">/dev/null") . " 2>$errfile.err");
   if ($rc) {
      die("shared module compilation failed; see the error log below\n\n" , `cat $errfile.err`);
   }
}

sub load_shared_module {
   my ($so_name)=@_;
   -f $so_name or die "shared module $so_name does not exist\n";
   my $timestamp=(stat _)[9];
   DynaLoader::dl_load_file($so_name, 0x01)
      or die "Can't load shared module $so_name: ", DynaLoader::dl_error(), "\n";
   $timestamp;
}

sub create_source_file {
   my ($self, $file, $template, %vars)=@_;
   $file =~ $filename_re;
   if (-f (my $tf="$InstallTop/lib/PTL/skel/$1")) {
      $template=$tf;
   } elsif (defined $template) {
      $template="$InstallTop/lib/PTL/skel/$template";
      unless (-f $template) {
	 die "Neither C++ source file $file nor its template $template exist:\n",
	     "Unable to maintain persistent C++ bindings!\n";
      }
   } else {
      die "C++ source file $file does not exist and there is no matching template in $InstallTop/lib/PTL/skel.\n",
	  "Unable to maintain persistent C++ bindings!\n";
   }

   local $/;
   open my $tf, $template
   or die "Could not read the source file template $template: $!\n",
	  "Unable to maintain persistent C++ bindings!\n";

   my $content=<$tf>;  close $tf;
   # don't parse the copyright notice at the file top
   $content =~ m|\*/$|m;
   $content = $` . $& . eval "<<\".#.#.#.\";\n$'.#.#.#.\n";
   if ($@) {
      die "Error processing the source file template $template: $@";
   }
   # kill superfluous empty lines
   $content =~ s/^\n{2,}/\n/m;

   open $tf, ">", $file
   or die "Could not create the C++ source file $file: $!\n",
	  "Unable to maintain persistent C++ bindings!\n";
   print $tf $content;  close $tf;
}

sub create_source_dir {
   my ($path)=@_;
   my @to_create;
   my $use_svn;
   while ($path =~ $filename_re) {
      $path=substr($path,0,$-[0]-1);
      if (-d $path) {
	 $use_svn=-d "$path/.svn";
	 last;
      } else {
	 push @to_create, $path;
      }
   }
   foreach (reverse @to_create) {
      if ($use_svn) {
	 my $svn_success=qx( svn mkdir $_ );
	 if ($svn_success !~ /^A /) {
	    die "Could not create directory $_ in your svn working copy.\n",
	        "Please investigate the reason and perform 'svn mkdir $_' manually!\n";
	 }
      } elsif (!mkdir($_, 0755)) {
	 die "Could not create directory $_: $!\n";
      }
   }
   $use_svn
}

sub create_wrapper_mirror {
   my $src_top=shift;
   my $dir;
   if ($src_top eq $InstallTop) {
      $dir="$UserSettings::dir/mirrors/core";
   } else {
      $src_top =~ $filename_re;
      my $cnt;
      while (-d ($dir="$UserSettings::dir/mirrors/$1".($cnt && ".$cnt"))) { ++$cnt }
   }
   warn_print( "Creating mirror tree $dir for automatically generated C++/perl wrappers" );
   create_source_dir($dir);
   $wrapper_mirrors{$src_top}=$dir;
   $custom_handler->set('%wrapper_mirrors');
   $dir
}

sub source_sanity_checks {
   my ($self, $file, @template)=@_;
   my $orig_file=$file;
   my ($install_top, $so_name, $so_timestamp, $so_mirrored)=$self->guess_file_location($orig_file);
   my ($mirror_top, $mirror_app);
   if ($so_mirrored) {
      my $app_name=$self->application->name;
      $mirror_top=$wrapper_mirrors{$install_top} || create_wrapper_mirror($install_top);
      $mirror_app="$mirror_top/apps/$app_name";
      unless ($file =~ s|^$install_top|$mirror_top|) {
	 # embedded functions are still carrying the original paths from the developers' working copies...
	 $file =~ s|^(.*)(?=/apps/$app_name/src/)|$mirror_top|;
      }
   }
   my $wrapper_tag=($install_top ne $self->application->installTop && "$install_top/").$self->application->name;

   if (-r $file && -w _) {
      my $file_t=(stat _)[9];
      if (defined($so_name) && $so_mirrored>=0 && $file_t > $so_timestamp && -f $so_name) {
	 $wrapper_updated{$wrapper_tag}=time+1;
	 $custom_handler->set('%wrapper_updated');
	 die <<".";
Automatical update of the C++ source file $file refused:
The shared module $so_name is out-of-date!
It will be recompiled at the very beginning of the next polymake session.
Alternatively, you may run `make' manually right now.
.
      }
      $file =~ $directory_re;
      if (-d "$1/.svn") {
	 my $svn_status=qx(svn status -u $file);
	 if ($svn_status =~ /^C /) {
	    die <<".";
The C++ source file $file is in conflicting state w.r.t. the checked-in version.
Please resolve the conflict and run `make' manually before you start polymake the next time.
.
	 } elsif ($svn_status =~ /\* /) {
	    die <<".";
The C++ source file $file is not up-to-date w.r.t. the repository.
Automatical update now would lead to future conflicts.
Please update your working copy and run `make' manually before you start polymake the next time.
.
	 } elsif ($svn_status !~ /\S/) {
	    warn_print( "Could not check the SVN status of file $file - probably no connection to the repository?\n",
			"Automatical updates are made to your own risk." );
	 }
      }

      dbg_print( "Updating C++ source file $file" ) if $Verbose::cpp;

   } elsif (-e _) {
      die <<"."
The C++ source file $file exists but can't be updated due to lacking permission.
Until this file is writable for you, you can't maintain persistent C++ bindings!
.
   } else {
      my $use_svn=create_source_dir($file);
      create_source_file($self,$file,@template,SVN=>$use_svn);
      if ($use_svn) {
	 my $svn_success=qx( svn add $file );
	 if ($svn_success !~ /^A /) {
	    warn_print( "Could not add $file to your svn working copy.\n",
			"Please investigate the reason and perform 'svn add $file' manually!\n" );
	 }
      }
      if ($so_mirrored) {
         if (@{$self->application->import_sorted} && !-f "$mirror_app/src/Makefile.inc") {
            open my $MF, ">$mirror_app/src/Makefile.inc";
            print $MF  "ImportApps := @{$self->application->import_sorted}\n";
            close $MF;
         }
	 if ($orig_file =~ s|/src/perl/wrap-|/src/|) {
	    my $link=$mirror_app.substr($orig_file,$-[0]);
	    symlink $orig_file, $link
	       or die "Could not create a link $link => $orig_file: $!\n";
	 }
      }
      if ($Verbose::cpp) {
	 dbg_print( "Creating C++ source file $file" );
      }
   }
   ( filename => $file, wrapper_tag => $wrapper_tag );
}

my $cpp_include_re=qr{^\s*\#\s*include\s+<(.*?)>};
my $cpp_start_namespace_re=qr{^\s*namespace polymake\b};

sub modify_source_file {
   my ($self, $what)=@_;
   my $filename=$what->{filename};
   open my $F, $filename;
   my ($Fnew, $F_k)=new OverwriteFile($filename);
   local $_;

   # copy the preamble till the first #include
   while (defined ($_=<$F>) && ! /(?:$cpp_include_re|$cpp_start_namespace_re)/o) {
      print $Fnew $_;
   }

   my $includes=$what->{includes};
   if (defined($includes) && @$includes) {
      # merge the include directives avoiding duplicates
      my @read_includes;
      while ($_ =~ $cpp_include_re) {
	 push @read_includes, $1;
	 $_=<$F>;
      }
      print $Fnew "#include <$_>\n" for (uniq(@read_includes, @$includes));
      print $Fnew "\n" unless @read_includes;
   }

   do {
      print $Fnew $_;
   } while (! /Automatically generated contents follow/ && defined ($_=<$F>));

   # insert new template declarations
   if (defined (my $declarations=$what->{declarations})) {
      print $Fnew @$declarations;
   }

   # delete obsolete template declarations
   if (defined (my $to_delete=$what->{to_delete})) {
      my %seen;
      while (<$F>) {
	 my $theader;
	 if (/^\s*template\s+</) {
	    $theader=$_;
	    $_=<$F>;
	 }
	 if (/^\s*FunctionInterface4perl\s*\(\s* ($id_re) /xo) {
	    if (defined (my $auto_func=$to_delete->{$1})) {
	       $seen{$auto_func}|=1;
	       do { $_=<$F> } while (/\S/);
	       next;
	    }
	 } elsif (!defined($theader) && /^\s*(?:Function|Operator)Instance4perl\s*\(\s* ($id_re) /xo) {
	    if (defined (my $auto_func=$to_delete->{$1})) {
	       $seen{$auto_func}|=2;
	       next;
	    }
	 } elsif (/Automatically generated contents end here/) {
	    last;
	 }
	 print $Fnew $theader, $_;
      }

      if (my @missed=grep { $seen{$_}!=3 } values %$to_delete) {
	 warn_print( "Wrapper declarations and/or instantiations of functions to be deleted from the source file $filename have not be found there:\n",
		     (map { ("  ", $_->perl_name, "\n") } @missed),
		     "Please edit the source code manually\n" );
      }
   } else {
      while (defined ($_=<$F>) && ! /Automatically generated contents end here/) {
	 print $Fnew $_;
      }
   }

   # insert new instances
   if (defined (my $instances=$what->{instances})) {
      print $Fnew @$instances;
   }

   # copy the trailer
   do { print $Fnew $_ } while (<$F>);
   close $F; close $Fnew;
}

sub init_h_file {
   my ($self, $src_top, $hfile)=@_;
   my $filename= $hfile eq "polymake.h" ? "$src_top/include/$hfile" : "$src_top/include/perl/bindings/$hfile";
   (my $name=$hfile) =~ s/\.h$//;
   $name =~ s{[./: ]}{_}g;
   { declarations => [ ],
     source_sanity_checks($self, $filename, "bindings.h", name=>$name, include_file=>$hfile) }
}

sub init_cc_file {
   my ($self, $filename)=@_;
   { declarations => [ ], includes => [ ], instances => [ ],
     source_sanity_checks($self, $filename, "bindings.cc") }
}

sub init_ext_makefile {
   my ($self, $dict, $extension)=@_;
   my $app_name=$self->application->name;
   my $ext_mf=$dict->{$extension->[0]->dir} ||= do {
      my $mf_name=$self->extensions->{$extension->[0]->dir}->[0];
      $mf_name =~ s|/lib/$app_name.*|/apps/$app_name/Makefile|;
      open my $MF, $mf_name or die "Makefile $mf_name lacking\n";
      my $imports=[ ];
      local $_;
      while (<$MF>) {
	 if (/^\s*ImportExts\s*:?=\s*/) {
	    @$imports=map { add Extension($_) } $' =~ /\S+/g; last;
	 }
      }
      { filename => $mf_name, imports => $imports, changed => 0 }
   };
   local_shift($extension);
   my $old=@{$ext_mf->{imports}};
   set_extension($ext_mf->{imports}, $extension);
   $ext_mf->{changed} ||= $old < @{$ext_mf->{imports}};
}

sub modify_ext_makefile {
   my ($self, $what)=@_;
   my $filename=$what->{filename};
   dbg_print( "Updating $filename" ) if $Verbose::cpp;

   open my $F, $filename;
   my ($Fnew, $F_k)=new OverwriteFile($filename);
   my $import="ImportExts := " . join(" ", map { $_->dir } @{$what->{imports}}) . "\n";
   local $_;
   while (<$F>) {
      if (/^\s*ImportExts\s*:?=\s*/) {
	 print $Fnew $import;  last;
      } elsif (/^\s*BuildDir\s*:?=/) {
	 print $Fnew $import, $_;  last;
      } else {
	 print $Fnew $_;
      }
   }
   while (<$F>) { print $Fnew $_; }
   close $F; close $Fnew;
}

sub generate_sources {
   my $self=shift;
   my (%files, %ext_mf);

   foreach my $proto (keys %{$self->all_lacking_types}) {
      my $opts=$proto->cppoptions->finalize;
      my $pkg= $proto->context_pkg || ($opts->builtin || $opts->special ? "builtins" : $proto->pkg);
      my $src_top=$opts->source_path($self->application);
      $pkg =~ /($id_re)$/o;
      my $source_file="$src_top/src/perl/$1.cc";
      my $ccfile=($files{$source_file} ||= init_cc_file($self,$source_file));
      push @{$ccfile->{includes}}, @{$opts->include};
      push @{$ccfile->{instances}}, $opts->register_class($proto->pkg);

      if (!$opts->template_params && $pkg ne "builtins") {
	 $source_file=$opts->include->[-1];
	 push @{ ($files{$source_file} ||= init_h_file($self,$src_top,$source_file))->{declarations} },
	      $opts->recognizing_class($proto->pkg);
      }

      init_ext_makefile($self, \%ext_mf, $opts->extension) if ref($opts->extension) eq "ARRAY";
   }

   while (my ($pkg, $generic_opts)=each %{$self->lacking_templates}) {
      my $source_file=$generic_opts->include->[-1];
      my $src_top=$generic_opts->source_path($self->application);
      push @{ ($files{$source_file} ||= init_h_file($self,$src_top,$source_file))->{declarations} },
	   $generic_opts->recognizing_template($pkg);
      $pkg =~ /($id_re)$/o;
      $source_file="$src_top/src/perl/$1.cc";
      my $ccfile=($files{$source_file} ||= init_cc_file($self,$source_file));
      push @{$ccfile->{includes}}, @{$generic_opts->include};
      push @{$ccfile->{instances}}, $generic_opts->register_template($pkg);
   }

   foreach my $inst (@{$self->lacking_auto_functions}) {
      my $source_file=$inst->source_file;
      my $ccfile=($files{$source_file} ||= init_cc_file($self,$source_file));
      push @{$ccfile->{includes}}, $inst->include;
      if (ref (my $seen_in=$inst->is_instance_of->seen_in)) {
	 unless ($seen_in->{$ccfile->{filename}}) {
	    $seen_in->{$ccfile->{filename}}=1;
	    push @{$ccfile->{declarations}}, $inst->is_instance_of->gen_source_code;
	 }
      }
      push @{$ccfile->{instances}}, $inst->gen_source_code;

      init_ext_makefile($self, \%ext_mf, $inst->extension) if ref($inst->extension) eq "ARRAY";
   }

   foreach my $auto_func (@{$self->obsolete_auto_functions}) {
      foreach (keys %{$auto_func->seen_in}) {
	 ($files{$_} ||= init_cc_file($self,$_))->{to_delete}->{$auto_func->wrapper_name}=$auto_func;
      }
   }

   %{$self->all_lacking_types}=();
   %{$self->lacking_templates}=();
   @{$self->lacking_auto_functions}=();
   @{$self->obsolete_auto_functions}=();

   my $t=time+1;
   foreach (values %files) {
      modify_source_file($self,$_);
      $wrapper_updated{$_->{wrapper_tag}}=$t;
   }
   foreach (values %ext_mf) {
      modify_ext_makefile($self,$_) if $_->{changed};
   }
   $custom_handler->set('%wrapper_updated');
}

sub generate_init_sources {
   my $self=shift;
   my $top_header=$self->application->top."/include/polymake.h";
   unless (-f $top_header) {
      source_sanity_checks($self,$top_header);
      ensure_update_sources($self);
   }
   $self->create_extension_build_dir($self->application->installTop, 1);
   1
}

sub ensure_update_sources {
   my $self=shift;
   add AtEnd($self->application->name.":C++", sub { generate_sources($self) },
	     before=>[ "Customize", "c++filt", map { "$_:C++" } keys(%{$self->application->used}) ]);
   $self->will_update_sources=1;
}

sub compile_load_temp_shared_module {
   my $self=shift;
   if (my @still_lacking=grep { !exists $root->classes->{$_->pkg} } keys %{$self->lacking_types} or @_) {
      local $_;
      my $extension;
      my $file=new Tempfile;
      foreach (@still_lacking) {
	 $_->cppoptions->finalize;
	 set_extension($extension, $_->cppoptions->extension) if defined($_->cppoptions->extension);
      }
      set_extension($extension, $_[0]->extension) if @_ && defined($_[0]->extension);

      my $debug_flag= $debug && "Debug=y";
      my $app_name=$self->application->name;

      open my $MF, ">$file.mk";
      print $MF "SharedModules := ", $file->basename, "\n",
                "ImportApps := ", join(" ", $app_name, @{$self->application->import_sorted}), "\n";

      if (defined $extension) {
	 if (is_object($extension)) {
	    print $MF "ExtensionTop := ", $extension->dir, "\n";
	 } else {
	    print $MF "ExtensionTop := ", $extension->[0]->dir, "\n",
	              "ImportExts := ", join(" ", map { $_->dir } @$extension[1..$#$extension]), "\n";
	 }
      } elsif ($self->application->installTop ne $InstallTop) {
	 print $MF "AppTop := ".$self->application->installTop."\n";
      }
      print $MF <<".";
BuildDir := .
ProjectTop := $InstallTop
SourceDir := .
include $InstallArch/conf.make
include \${ProjectTop}/support/build.make
.
      close $MF;

      my $tried=0;
      my $so_name=$file->rename.$dl_suffix;
      my ($shared)= $so_name =~ $filename_re;
    RETRY: {
	 open my $cc, ">$file.cc";
	 print $cc <<".";
#include <unistd.h>
namespace { void delete_temp_file() __attribute__((destructor));
	    void delete_temp_file() { unlink("$so_name"); } }
.
	 print $cc "#include <$_>\n"
            for ( uniq((map { @{$_->cppoptions->include} } @still_lacking),
		       (map { @{$_->include} } values %{$self->lacking_templates}),
		       (map { $_->temp_include($self) } @_)),
		  "polymake.h" );

	 print $cc "namespace polymake { namespace ".$self->application->name." {\n",
                   ( map { $_->cppoptions->register_class($_->pkg) } @still_lacking ),
		   ( map { $_->gen_temp_code } @_ ),
		   "} }\n";
		   
	 close $cc;

	 warn_print( "Compiling temporary shared module, please be patient..." ) if $Verbose::cpp;
	 my $rc=system(($Verbose::cpp>1 && "cat $file.cc >&2; ") .
		       "make -C " . $file->dir . " -f $file.mk $shared $debug_flag OwnShared=$shared ProcessDep=none " .
		       ($Verbose::cpp ? ">&2 " : ">/dev/null") . " 2>$file.err");
	 if ($rc) {
	    if (@_ ? ! map { $_->give_up } @_ : !$tried++ && !@still_lacking && ! keys %{$self->lacking_templates}) {
	       if ($Verbose::cpp) {
		  dbg_print( "Compilation failed:\n\n" . `cat $file.err` . "\n\nRetrying with alternative source code." )
	       }
	       redo RETRY;
	    }
	    die "Shared module compilation failed; see the error log below\n\n" . `cat $file.err`;
	 }
      }

      my $functions_begin=@{$root->functions};
      load_shared_module($so_name);

      foreach (@still_lacking) {
	 unless ($root->classes->{$_->pkg}) {
	    die " C++ binding of type ", $_->full_name, " not found where expected\n";
	 }
      }
      $file =~ $filename_re;
      bind_functions($self,$functions_begin,$#{$root->functions},
		     { @_ && defined($_[0]->is_instance_of->embedded) ? ("$1.cc" => $_[0]->is_instance_of->embedded) : () });
   }
   %{$self->lacking_types}=();
}

#######################################################################################

my ($cppf_out, $cppf_in);

sub demangle {
   if (!$cppf_in) {
      ($cppf_out, $cppf_in)=new BackgroundPipe($cppfilt_util);
      add AtEnd("c++filt", sub { close $cppf_out; close $cppf_in }, before=>"BackgroundProcess");
   }

   my $case=0;
   foreach my $rule (@to_mangled_defconstr) {
      local $_=$_[0];
      eval $rule or $@ ? die "syntax error in mangling regex: $@\n" : next;
      print $cppf_out $_, "\n";
      if (++$case<=2) {
	 if (readline($cppf_in) =~ $constructor_re) {
	    return $`;
	 }
      } else {
	 if (readline($cppf_in) =~ $wrapped_constructor_re) {
	    return $1;
	 }
      }
   }
   undef
}

#######################################################################################

sub gen_proxy_op {
   eval <<".";
sub {
   if (\$_[2]) {
      \$_[1] $_ convert_to_$_[0](\$_[0])
   } else {
      convert_to_$_[0](\$_[0]) $_ \$_[1]
   }
}
.
}

#######################################################################################

package Polymake::Core::CPlusPlus::Iterator;

use overload '++' => \&incr, 'bool' => \&not_at_end,	
             '${}' => \&deref_to_scalar, '@{}' => \&deref, '%{}' => \&deref,
             '=' => \&overload_clone_op,
             fallback => 0, nomethod => \&missing_op, '""' => sub { "ITERATOR" };


#######################################################################################

package Polymake::Core::CPlusPlus::NumProxy;

use overload '0+' => \&convert_to_primitive,
             neg => sub { - convert_to_primitive($_[0]) },
             abs => sub { abs( convert_to_primitive($_[0]) ) },
             (map { $_ => gen_proxy_op("primitive") } qw(+ - * / % ** <=>));

#######################################################################################

package Polymake::Core::CPlusPlus::BoolProxy;

use overload 'bool' => \&convert_to_primitive,
             (map { $_ => gen_proxy_op("primitive") } qw(& | ^));

#######################################################################################

package Polymake::Core::CPlusPlus::StringProxy;

use overload '""' => \&convert_to_string,
             (map { $_ => gen_proxy_op("string") } qw(x . cmp));


1
