#  Copyright (c) 1997-2009
#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
#  http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 2, or (at your option) any
#  later version: http://www.gnu.org/licenses/gpl.txt.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#-----------------------------------------------------------------------------
#  $Project: polymake $$Id: java.make 9089 2009-02-19 17:37:14Z gawrilow $

.PHONY: all all-deps all-own all-native all-native-h clean install install-native

### provide 'all' be a default target
all : all-deps all-own all-native

include ${ProjectTop}/support/utils.make
include ${BuildDir}/conf.make

### locate the sources
PackageName := $(notdir ${CURDIR})

JavaBuildDir := ${ProjectTop}/java_build
ClassBuildDir := ${JavaBuildDir}/${PackageBase}/${PackageName}

ifndef SourceDir
  SourceDir := $(wildcard ${ProjectTop}/java/src/${PackageBase}/${PackageName})
endif
ifndef SourceDir
  $(error SourceDir undefined, cannot locate source files)
endif

-include ${SourceDir}/Makefile.inc

Jars := ${JavaBuildDir} $(patsubst %,${JavaBuildDir}/%.jar, ${ImportApps})
JarFile := ${JavaBuildDir}/${PackageName}.jar

ifdef NativeClasses
  NativeLib := ${BuildDir}/lib/lib${PackageName}-JNI$(if ${Debug},-d).$(subst bundle,jnilib,${SO})
  NativeClassHeaders := $(addprefix ${ClassBuildDir}/,$(addsuffix .h, ${NativeClasses}))
endif

###
ifdef JavaBuild
  IgnoreSources := $(foreach f,$(call _expand_var,JavaIgnore),$(if $(findstring .,$f),$f,$f.java))
  Sources := $(filter-out $(addprefix %/,${IgnoreSources}) $(call _get_no_export,${SourceDir},broken|test), \
                          $(wildcard ${SourceDir}/*.java))
  Classes := $(patsubst %.java, %.class, $(notdir ${Sources}))
  OtherFiles := $(filter-out $(call _expand_var,JavaIgnore),${OtherFiles})

  ifdef ImportApps
    ifdef BeFirst
      define _MakeDependence
	$(MAKE) --print-directory -C ${JavaBuildDir}/${PackageBase}/$(1) BeFirst= $(2)

      endef
      define _MakeDependences
	$(foreach m,${ImportApps},$(call _MakeDependence,$m,$(1)))
      endef
    endif
  endif
endif

ifndef _MakeDependences
  _MakeDependences := : ;
endif


ifdef Sources
  all-own : ${JarFile}

  ${JarFile} : ${Sources} $(addprefix ${SourceDir}/, ${OtherFiles})
	${JAVAC} ${JAVACFLAGS} -d ${JavaBuildDir} -classpath $(call _list2path,${CLASSPATH} ${Jars} ${AddJars}) ${Sources}
	$(if ${OtherFiles},cp $(addprefix ${SourceDir}/, ${OtherFiles}) .)
	cd ${JavaBuildDir}; ${JAR} cvf $(notdir ${JarFile}) \
		 $(addprefix ${PackageBase}/${PackageName}/, $(patsubst %.class,%[.\$$]*class,${Classes}) ${OtherFiles})

else
  all-own : ;@:
endif

all-deps:
	+@$(call _MakeDependences,all)

ifdef NativeClasses
  define _MakeNative
	$(MAKE) --print-directory -C ${BuildDir}/apps/${PackageName} $(1) SourceDir='$${ProjectTop}/apps/${PackageName}/java' \
		SharedModules="${NativeClasses}" OwnShared='$${BuildDir}/lib/$(notdir ${NativeLib})' BeFirst= \
		ExtraCFLAGS="-I${CURDIR} $(addprefix -I,${JNI_PATH} $(dir $(wildcard ${JNI_PATH}/*/jni_md.h)))" Libs= CCLD='$${CC}'

  endef

  all-native : all-native-h
	@+$(call _MakeNative,all-lib)

  all-native-h : ${NativeClassHeaders}

  ${NativeClassHeaders} : ${ClassBuildDir}/%.h : ${SourceDir}/%.java
	cd ${JavaBuildDir}; \
	${JAVAH} -o ${PackageBase}/${PackageName}/$(notdir $@) $(if ${JavaBuild},,-classpath ${PackageName}.jar) \
		 $(subst /,.,${PackageBase}).${PackageName}.$*
	@touch $@

  install-native : all-native
	${PERL} ${INSTALL_PL} -m 444 ${NativeLib} ${InstallArch}/lib
else
  all-native install-native : ;@:
endif

### delete all non-sources
clean:
  ifdef JavaBuild
	rm -f *.class ${JarFile} ${OtherFiles}
  endif
  ifdef NativeClasses
	rm -f ${NativeClassHeaders} ${NativeLib} $(patsubst %,${BuildDir}/apps/${PackageName}/%.o,${NativeClasses})
  endif
	@+$(call _MakeDependences,clean)

InstallJavaDir := ${InstallTop}/jars

install : all
  ifdef BeFirst
	${PERL} ${INSTALL_PL} -d -m ${DirMask} -U ${InstallJavaDir}
  endif
	${PERL} ${INSTALL_PL} -m 444 ${JarFile} ${InstallJavaDir}
	+@$(call _MakeDependences,install)


# Local Variables:
# mode: Makefile
# End:
